package be.ulg.hoessen.martinLeon;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * Permet d'avoir des méthodes que l'on utilisera dans les classes de test
 * @author benjamin, bhoessen
 */
public class Util {
    
    public static boolean console = false;
    public static boolean verbose = false;

    public static void message(String message) {
        if(console){
            System.out.println(message);
        } else {
            JOptionPane.showMessageDialog(null, message);
        }
    }
    
    /**
     * Permet de basculer en mode console
     */
    public static void modeConsole(){
        console = true;
    }
    
    /**
     * Permet de mettre le mode interface utilisateur
     */
    public static void modeIU(){
        console = false;
    }
    
    /**
     * Prévient l'utilisateur qu'une erreur s'est produite.
     * @param message le message d'erreur.
     */
    public static void erreur(String message){
        if(!console){
            JOptionPane.showMessageDialog( null,message,
                    "Erreur",JOptionPane.ERROR_MESSAGE);
        } else {
            System.out.println("Erreur:");
            System.out.println(message);
        }
    }
    
    /**
     * Permet de lire les entrées de l'utilisateur
     * @param message la question que l'on pose à l'utilisateur
     * @return la réponse de l'utilisateur
     */
    public static String lire(String message){
        if(!console){
            return JOptionPane.showInputDialog(null,message);
        } else {
            System.out.println(message);
            String ligneLue="";
            try{
                InputStreamReader isr = new InputStreamReader(System.in);
                BufferedReader br = new BufferedReader(isr);
                ligneLue = br.readLine();
            } catch(IOException ex){
                System.out.println(ex.toString());
            }
            return ligneLue;
        }
    }
    
    /**
     * Permet de choisir un fichier sur lequel on écrira le résultat, et crée
     * le fichier si celui-ci n'existe pas.
     * @return le fichier donné.
     */
    public static File choisirFichier(){
        if(!console){
            JFileChooser interfaceUtilisateur = new JFileChooser();
            interfaceUtilisateur.showSaveDialog(null);
            return interfaceUtilisateur.getSelectedFile();
        } else{
            String chemin = lire("Quel est le chemin du fichier sur lequel " +
                    "vous voulez enregistrer?");
            return new File(chemin);
        }
        
    }

    /**
     * Passer en mode verbeux
     */
    public static void modeVerbeux() {
        verbose = true;
    }
    
}
