/*
 * Automate.java
 *
 * Created on 7 octobre 2007, 21:31
 *
 */
package be.ulg.hoessen.martinLeon.chargeurs;

import be.ulg.hoessen.martinLeon.traitement.*;
import java.util.HashSet;

/**
 * Classe definissant la machine d'etat.
 * @author benoit, benjamin
 */
public class Automate {

    private ChargeurTexte chargeur;

    // variable representant l'etat de la machine
    private int etat = MOT;
    private int etatPrincipal = TEXTE;

    // variables permettant de gérer les commentaires
    private boolean commentaire = false;
    private int nbCom = 0;

    // valeurs des differents etats
    private final static int BALISE = 10,  BALISE_META = 11,  BALISE_META_C = 111,  BALISE_META_CHARSET = 112,  BALISE_A = 12,  BALISE_A_H = 121,  BALISE_A_HREF = 122,  BALISE_A_URL = 123,  BALISE_A_URL_ENTITE = 124,  BALISE_AUTRE = 13,  BALISE_NOM = 15,  TEXTE = 20,  MOT_ACCENT = 21,  MOT = 23;

    // tampon utilise pour le stockage des mots et url
    private StringBuffer tampon;

    // tampon utilise pour le stockage des 4 prochain caractères à traiter
    private StringBuffer cTampon;

    // tampon utilise pour le stockage des entités html
    private StringBuffer entite;
    /** Donne la liste des caracteres de separation de mots */
    private static HashSet<Character> separateurs;

    public Automate(ChargeurTexte chargeur) {
        this.chargeur = chargeur;
        tampon = new StringBuffer();
        cTampon = new StringBuffer("    ");
        entite = new StringBuffer();
        initialiserSeparateurs();
    }

    public void reinitialiser() {
        tampon = new StringBuffer();
        cTampon = new StringBuffer("    ");
        entite = new StringBuffer();
    }

    /**
     * Methode qui permet d'executer l'(les) instruction(s) correspondant a un
     * etat donne.
     */
    public void instructionEtat(char caractere, boolean ignorer) {
        caractere = ChangeurEntite.caractereCorrespondant(caractere);
        char carTmp = ' ';

        // ajout du caractere courant dans le tampon
        if (cTampon.length() < 4) {
            cTampon.append(caractere);
        } else {
            carTmp = cTampon.charAt(0);
            cTampon.replace(0, 3, cTampon.substring(1, 4));
            cTampon.setCharAt(3, caractere);
        }

        // tampon pas encore plein
        if (cTampon.length() < 4) {
            commentaire = false;
        }

        // tampon plein
        if (cTampon.length() == 4) {
            // début de commentaire
            if (cTampon.toString().compareTo("<!--") == 0) {
                nbCom++;
                commentaire = true;
            }
            // fin de commentaire
            if (cTampon.substring(1, 4).compareTo("-->") == 0) {
                nbCom--;
                commentaire = (nbCom > 0);
            }
        }
        caractere = carTmp;

        if (!commentaire) {
            switch (etatPrincipal) {
                case BALISE:
                    //test pour verifier que l'on est pas a la fin d'une balise
                    if (caractere == '>') {
                        if (etat == BALISE_A_URL && tampon.length() > 0) {
                            if (!ignorer) {
                                chargeur.envoieURL(tampon.toString());
                            }
                        }
                        etatPrincipal = TEXTE;
                        etat = MOT;
                        tampon.delete(0, tampon.length());
                    //System.out.println("Mot");
                    } else {
                        switch (etat) {
                            case BALISE_NOM:
                                if (caractere != ' ') {
                                    tampon.append(caractere);
                                } else {
                                    String nom = tampon.toString();
                                    if (nom.equalsIgnoreCase("meta")) {
                                        etat = BALISE_META;
                                        tampon.delete(0, tampon.length());
                                    } else if (nom.equalsIgnoreCase("a")) {
                                        //System.out.println("Balise lien trouvee");
                                        etat = BALISE_A;
                                        tampon.delete(0, tampon.length());
                                    } else {
                                        etat = BALISE_AUTRE;
                                    //System.out.println("Balise non prise en compte: "+nom);
                                    }
                                }
                                break;
                            case BALISE_META:
                                if (caractere == 'c') {
                                    etat = BALISE_META_C;
                                    tampon.append('c');
                                }
                                if (caractere == 'C') {
                                    etat = BALISE_META_C;
                                    tampon.append('C');
                                }
                                break;
                            case BALISE_META_C:
                                if ((caractere != ' ') && (caractere != '=')) {
                                    tampon.append(caractere);
                                } else {
                                    String c = tampon.toString();
                                    if (c.equalsIgnoreCase("charset")) {
                                        etat = BALISE_META_CHARSET;
                                        tampon.delete(0, tampon.length());
                                    } else {
                                        etat = BALISE_META;
                                        tampon.delete(0, tampon.length());
                                    }
                                }
                                break;
                            case BALISE_META_CHARSET:
                                if ((caractere != '"') && (caractere != ' ')) {
                                    tampon.append(caractere);
                                } else {
                                    etat = BALISE_AUTRE;
                                    if (ignorer) {
                                        chargeur.changerCharset(tampon.toString());
                                    }
                                    tampon.delete(0, tampon.length());
                                }
                                break;
                            case BALISE_A:
                                if (caractere == 'h') {
                                    etat = BALISE_A_H;
                                    if (tampon.length() > 0) {
                                        tampon.delete(0, tampon.length());
                                    }
                                    tampon.append('h');
                                } else if (caractere == 'H') {
                                    etat = BALISE_A_H;
                                    if (tampon.length() > 0) {
                                        tampon.delete(0, tampon.length());
                                    }
                                    tampon.append('H');
                                } else {

                                }
                                break;
                            case BALISE_A_H:
                                if ((caractere != ' ') && (caractere != '=')) {
                                    tampon.append(caractere);
                                } else {
                                    String h = tampon.toString();
                                    if (h.equalsIgnoreCase("href")) {
                                        etat = BALISE_A_HREF;
                                        tampon.delete(0, tampon.length());
                                    } else {
                                        etat = BALISE_A;
                                        tampon.delete(0, tampon.length());
                                    }
                                }
                                break;
                            case BALISE_A_HREF:
                                if ((caractere != '=') && (caractere != ' ') && (caractere != '"')) {
                                    etat = BALISE_A_URL;
                                    //System.out.print('\n');
                                    //System.out.print(caractere);
                                    tampon.append(caractere);
                                } else {

                                }
                                break;
                            case BALISE_A_URL:
                                //System.out.print(caractere);
                                if ((caractere != '"') && (caractere != ' ') && caractere != '#') {
                                    if (caractere == '&') {
                                        etat = BALISE_A_URL_ENTITE;
                                    } else {
                                        tampon.append(caractere);
                                    }
                                } else {
                                    etat = BALISE_AUTRE;
                                    if (!ignorer) {
                                        chargeur.envoieURL(tampon.toString());
                                    }
                                }
                                break;
                            case BALISE_A_URL_ENTITE:
                                if (caractere != ';') {
                                    if ((caractere == '"') || (caractere == ' ')) {
                                        tampon.append(entite);
                                        entite.delete(0, entite.length());
                                        etat = BALISE_AUTRE;
                                        if (!ignorer) {
                                            chargeur.envoieURL(tampon.toString());
                                        }
                                    } else {
                                        entite.append(caractere);
                                    }
                                } else {
                                    char tmp = ChangeurEntite.caractereCorrespondant(entite.toString());
                                    entite.delete(0, entite.length());
                                    //System.out.print(tmp+"|");
                                    tampon.append(tmp);
                                    etat = BALISE_A_URL;
                                }
                                break;
                            case BALISE_AUTRE: // caractere ininteressant
                                break;
                        }
                    }
                    break;
                case TEXTE:
                    // instruction(s)
                    if (caractere == '<') {
                        if (tampon.length() > 0) {
                            //System.out.println(tampon);
                            if (!ignorer) {
                                chargeur.envoiMot(tampon.toString());
                            }
                            tampon.delete(0, tampon.length());
                        }
                        if (entite.length() > 0) {
                            entite.delete(0, entite.length());
                        }
                        etatPrincipal = BALISE;
                        etat = BALISE_NOM;
                    //System.out.println("Balise");
                    } else {
                        switch (etat) {
                            case MOT:
                                if (separateurs.contains(caractere)) {
                                    if (tampon.length() > 0) {
                                        if (!ignorer) {
                                            chargeur.envoiMot(tampon.toString());
                                        }
                                        tampon.delete(0, tampon.length());
                                    }
                                } else {
                                    if (caractere != '&') {
                                        tampon.append(caractere);
                                    } else {
                                        etat = MOT_ACCENT;
                                    }
                                }
                                break;
                            case MOT_ACCENT:
                                // instruction(s)
                                if (caractere != ';') {
                                    entite.append(caractere);
                                } else {
                                    char tmp = ChangeurEntite.caractereCorrespondant(entite.toString());
                                    entite.delete(0, entite.length());
                                    if (separateurs.contains(tmp)) {
                                        if (tampon.length() > 0) {
                                            if (!ignorer) {
                                                chargeur.envoiMot(tampon.toString());
                                            }
                                            tampon.delete(0, tampon.length());
                                        }
                                    } else {
                                        tampon.append(tmp);
                                    }
                                    etat = MOT;
                                }
                                break;
                        }
                    }
                    break;
            }
        }
    }

    public void trouverCharset(char caractere) {
        instructionEtat(caractere, true);
    }

    public void instructionEtat(char caractere) {
        instructionEtat(caractere, false);
    }

    private void initialiserSeparateurs() {
        if (separateurs == null) {
            separateurs = new HashSet<Character>();
            separateurs.add('{');
            separateurs.add('}');
            separateurs.add('(');
            separateurs.add(')');
            separateurs.add('[');
            separateurs.add(']');
            separateurs.add('-');
            separateurs.add('_');
            separateurs.add('+');
            separateurs.add('*');
            separateurs.add('/');
            separateurs.add('=');
            separateurs.add('!');
            separateurs.add('.');
            separateurs.add('?');
            separateurs.add('"');
            separateurs.add('\'');
            separateurs.add('\n');
            separateurs.add('\r');
            separateurs.add('\t');
            separateurs.add(';');
            separateurs.add(':');
            separateurs.add(',');
            separateurs.add(' ');
            separateurs.add('|');
        }
    }
}
