package be.ulg.hoessen.martinLeon.chargeurs;

import java.util.Hashtable;

/**
 * Le but de cette classe est de transformer la plupart des entites html
 * en une lettre non-accentuee.
 * @author benjamin, benoit
 */
public abstract class ChangeurEntite {
    
    private static Hashtable<Character,Character> caractereTable;
    private static Hashtable<String,Character> entiteesTable;
    
    /**
     * Methode permettant de renvoyer le caractere correspondant a celui passe
     * en parametre mais non-accentue
     * @param le caractere accentue
     * @return l'equivalent non-accentue, ou renvoie le parametre si aucune
     * correspondance n'est trouvee
     */
    public static char caractereCorrespondant(char caractere){
        if(caractereTable == null){
            initialiserCaractereTable();
        }
        Character car = caractereTable.get(caractere);
        if (car==null){
            return caractere;
        } else{
            //System.out.println(caractere+" => "+car.charValue());
            return car.charValue();
        }
    }
    
    /**
     * Methode permettant de renvoyer le caractere correspondant a l'entitee
     * XHTML passee en parametre
     * @param l'entitee XHTML, que ce soit son nom, son numero decimal prefixe
     * de # ou son numero hexadecimal prefixe de #x
     * @return l'equivalent non-accentue, ou ' ' si aucune correspondance n'est
     * trouvee
     */
    public static char caractereCorrespondant(String entite){
        if(entiteesTable == null){
            initialiserEntiteesTable();
        }
        Character car = entiteesTable.get(entite);
        if (car==null){
            return ' ';
        } else{
            return car;
        }
    }
    
    /**
     * Initialise la table de caracteres
     */
    private static void initialiserCaractereTable() {
        caractereTable = new Hashtable<Character,Character>(95);
        //On ajoute le "c"
        caractereTable.put('\u00E7','c');
        caractereTable.put('\u00C7','C');
        //On ajoute les "e"
        caractereTable.put('\u00E8','e');caractereTable.put('\u00EA','e');
        caractereTable.put('\u00E9','e');caractereTable.put('\u00EB','e');
        caractereTable.put('\u00C8','e');caractereTable.put('\u00C9','e');
        caractereTable.put('\u00CA','e');caractereTable.put('\u00CB','e');
        //On ajoute les "a"
        caractereTable.put('\u00E0','a');caractereTable.put('\u00E1','a');
        caractereTable.put('\u00E2','a');caractereTable.put('\u00E3','a');
        caractereTable.put('\u00E4','a');caractereTable.put('\u00E5','a');
        caractereTable.put('\u00C0','A');caractereTable.put('\u00C1','A');
        caractereTable.put('\u00C2','A');caractereTable.put('\u00C3','A');
        caractereTable.put('\u00C4','a');caractereTable.put('\u00C5','A');
        //On ajoute les "i"
        caractereTable.put('\u00EC','i');caractereTable.put('\u00ED','i');
        caractereTable.put('\u00EE','i');caractereTable.put('\u00EF','i');
        caractereTable.put('\u00CC','I');caractereTable.put('\u00CD','I');
        caractereTable.put('\u00CE','I');caractereTable.put('\u00CF','I');
        //On ajoute les "o"
        caractereTable.put('\u00F2','o');caractereTable.put('\u00F3','o');
        caractereTable.put('\u00F4','o');caractereTable.put('\u00F5','o');
        caractereTable.put('\u00F6','o');caractereTable.put('\u00F8','o');
        caractereTable.put('\u00D2','O');caractereTable.put('\u00D3','O');
        caractereTable.put('\u00D4','O');caractereTable.put('\u00D5','O');
        caractereTable.put('\u00D6','O');caractereTable.put('\u00D8','O');
        //On ajoute les "u"
        caractereTable.put('\u00F9','u');caractereTable.put('\u00FA','u');
        caractereTable.put('\u00FB','u');caractereTable.put('\u00FC','u');
        caractereTable.put('\u00D9','U');caractereTable.put('\u00DA','U');
        caractereTable.put('\u00DB','U');caractereTable.put('\u00DC','U');
    }
    
    /**
     * Initialise la table des entitees
     */
    private static void initialiserEntiteesTable() {
        entiteesTable = new Hashtable<String,Character>(220);
        //On ajoute les entites selon leur noms
        //On ajoute le 'c'
        entiteesTable.put( "Ccedil",'C');
        entiteesTable.put( "ccedil",'c');
        //On ajoute les 'a'
        entiteesTable.put( "Agrave",'A');
        entiteesTable.put( "Aacute",'A');
        entiteesTable.put( "Acirc",'A');
        entiteesTable.put( "Atilde",'A');
        entiteesTable.put( "Auml",'A');
        entiteesTable.put( "Aring",'A');
        entiteesTable.put( "AElig",'A');
        entiteesTable.put( "agrave",'a');
        entiteesTable.put( "aacute",'a');
        entiteesTable.put( "acirc",'a');
        entiteesTable.put( "atilde",'a');
        entiteesTable.put( "auml",'a');
        entiteesTable.put( "aring",'a');
        entiteesTable.put( "aElig",'a');
        //On ajoute les 'e'
        entiteesTable.put( "Egrave",'E');
        entiteesTable.put( "Eacute",'E');
        entiteesTable.put( "Ecirc",'E');
        entiteesTable.put( "Euml",'E');
        entiteesTable.put( "egrave",'e');
        entiteesTable.put( "eacute",'e');
        entiteesTable.put( "ecirc",'e');
        entiteesTable.put( "euml",'e');
        //On ajoute les 'i'
        entiteesTable.put( "Igrave",'I');
        entiteesTable.put( "Iacute",'I');
        entiteesTable.put( "Icirc",'I');
        entiteesTable.put( "Iuml",'I');
        entiteesTable.put( "igrave",'i');
        entiteesTable.put( "iacute",'i');
        entiteesTable.put( "icirc",'i');
        entiteesTable.put( "iuml",'i');
        //On ajoute les 'o'
        entiteesTable.put( "Ograve",'O');
        entiteesTable.put( "Oacute",'O');
        entiteesTable.put( "Ocirc",'O');
        entiteesTable.put( "Otilde",'O');
        entiteesTable.put( "Ouml",'O');
        entiteesTable.put( "ograve",'o');
        entiteesTable.put( "oacute",'o');
        entiteesTable.put( "ocirc",'o');
        entiteesTable.put( "otilde",'o');
        entiteesTable.put( "ouml",'o');
        //On ajoute les 'u'
        entiteesTable.put( "Ugrave",'U');
        entiteesTable.put( "Uacute",'U');
        entiteesTable.put( "Ucirc",'U');
        entiteesTable.put( "Uuml",'U');
        entiteesTable.put( "ugrave",'u');
        entiteesTable.put( "uacute",'u');
        entiteesTable.put( "ucirc",'u');
        entiteesTable.put( "uuml",'u');
        //On ajoute quelques symboles
        entiteesTable.put( "amp",'&');
        entiteesTable.put( "quot",'"');
        entiteesTable.put( "lt",'<');
        entiteesTable.put( "gt",'>');
        entiteesTable.put( "apos",'\'');
        
        //On ajoute les entites selon leur numero decimal
        //On ajoute le 'c'
        entiteesTable.put( "#199",'C');
        entiteesTable.put( "#231",'c');
        //On ajoute les 'a'
        entiteesTable.put( "#192",'A');
        entiteesTable.put( "#193",'A');
        entiteesTable.put( "#194",'A');
        entiteesTable.put( "#195",'A');
        entiteesTable.put( "#196",'A');
        entiteesTable.put( "#197",'A');
        entiteesTable.put( "#198",'A');
        entiteesTable.put( "#224",'a');
        entiteesTable.put( "#225",'a');
        entiteesTable.put( "#226",'a');
        entiteesTable.put( "#227",'a');
        entiteesTable.put( "#228",'a');
        entiteesTable.put( "#229",'a');
        entiteesTable.put( "#230",'a');
        //On ajoute les 'e'
        entiteesTable.put( "#200",'E');
        entiteesTable.put( "#201",'E');
        entiteesTable.put( "#202",'E');
        entiteesTable.put( "#203",'E');
        entiteesTable.put( "#232",'e');
        entiteesTable.put( "#233",'e');
        entiteesTable.put( "#234",'e');
        entiteesTable.put( "#235",'e');
        //On ajoute les 'i'
        entiteesTable.put( "#204",'I');
        entiteesTable.put( "#205",'I');
        entiteesTable.put( "#206",'I');
        entiteesTable.put( "#207",'I');
        entiteesTable.put( "#236",'i');
        entiteesTable.put( "#237",'i');
        entiteesTable.put( "#238",'i');
        entiteesTable.put( "#239",'i');
        //On ajoute les 'o'
        entiteesTable.put( "#210",'O');
        entiteesTable.put( "#211",'O');
        entiteesTable.put( "#212",'O');
        entiteesTable.put( "#213",'O');
        entiteesTable.put( "#214",'O');
        entiteesTable.put( "#242",'o');
        entiteesTable.put( "#243",'o');
        entiteesTable.put( "#244",'o');
        entiteesTable.put( "#245",'o');
        entiteesTable.put( "#246",'o');
        //On ajoute les 'u'
        entiteesTable.put( "#217",'U');
        entiteesTable.put( "#218",'U');
        entiteesTable.put( "#219",'U');
        entiteesTable.put( "#220",'U');
        entiteesTable.put( "#249",'u');
        entiteesTable.put( "#250",'u');
        entiteesTable.put( "#251",'u');
        entiteesTable.put( "#252",'u');
        //On ajoute quelques symboles
        entiteesTable.put( "#38",'&');
        entiteesTable.put( "#34",'"');
        entiteesTable.put( "#60",'<');
        entiteesTable.put( "#62",'>');
        entiteesTable.put( "#39",'\'');
        
        //On ajoute les entites selon leur numero hexadecimal
        //On ajoute le 'c'
        entiteesTable.put( "#xC7",'C');
        entiteesTable.put( "#xE7",'c');
        //On ajoute les 'a'
        entiteesTable.put( "#xC0",'A');
        entiteesTable.put( "#xC1",'A');
        entiteesTable.put( "#xC2",'A');
        entiteesTable.put( "#xC3",'A');
        entiteesTable.put( "#xC4",'A');
        entiteesTable.put( "#xC5",'A');
        entiteesTable.put( "#xC6",'A');
        entiteesTable.put( "#xE0",'a');
        entiteesTable.put( "#xE1",'a');
        entiteesTable.put( "#xE2",'a');
        entiteesTable.put( "#xE3",'a');
        entiteesTable.put( "#xE4",'a');
        entiteesTable.put( "#xE5",'a');
        entiteesTable.put( "#xE6",'a');
        //On ajoute les 'e'
        entiteesTable.put( "#xC8",'E');
        entiteesTable.put( "#xC9",'E');
        entiteesTable.put( "#xCA",'E');
        entiteesTable.put( "#xCB",'E');
        entiteesTable.put( "#xE8",'e');
        entiteesTable.put( "#xE9",'e');
        entiteesTable.put( "#xEA",'e');
        entiteesTable.put( "#xEB",'e');
        //On ajoute les 'i'
        entiteesTable.put( "#xCC",'I');
        entiteesTable.put( "#xCD",'I');
        entiteesTable.put( "#xCE",'I');
        entiteesTable.put( "#xCF",'I');
        entiteesTable.put( "#xEC",'i');
        entiteesTable.put( "#xED",'i');
        entiteesTable.put( "#xEE",'i');
        entiteesTable.put( "#xEF",'i');
        //On ajoute les 'o'
        entiteesTable.put( "#xD2",'O');
        entiteesTable.put( "#xD3",'O');
        entiteesTable.put( "#xD4",'O');
        entiteesTable.put( "#xD5",'O');
        entiteesTable.put( "#xD6",'O');
        entiteesTable.put( "#xF2",'o');
        entiteesTable.put( "#xF3",'o');
        entiteesTable.put( "#xF4",'o');
        entiteesTable.put( "#xF6",'o');
        entiteesTable.put( "#xF7",'o');
        //On ajoute les 'u'
        entiteesTable.put( "#xD9",'U');
        entiteesTable.put( "#xDA",'U');
        entiteesTable.put( "#xDB",'U');
        entiteesTable.put( "#xDC",'U');
        entiteesTable.put( "#xF9",'u');
        entiteesTable.put( "#xFA",'u');
        entiteesTable.put( "#xFB",'u');
        entiteesTable.put( "#xFC",'u');
        //On ajoute quelques symboles
        entiteesTable.put( "#x26",'&');
        entiteesTable.put( "#x22",'"');
        entiteesTable.put( "#x3C",'<');
        entiteesTable.put( "#x3E",'>');
        entiteesTable.put( "#x27",'\'');
    }
    
}
