package be.ulg.hoessen.martinLeon.chargeurs;

import be.ulg.hoessen.martinLeon.Util;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

/**
 * Classe permettant de charger un fichier de type texte et de l'envoyer à un Traiteur
 * @author benoit, benjamin
 */
public class ChargeurTexte extends Chargeur {

    private InputStreamReader lecteurFlux;
    private BufferedReader lecteur;
    private InputStream flux;
    private Automate automate;
    private URL urlPageCourante;
    private URLConnection urlConnection;
    private boolean charsetTrouve;
    
    /**
     * Cree un nouveau chargeur de fichier texte
     * @param url l'url du fichier a charger
     * @throw IOException si on ne peut ouvrir un flux depuis cet url
     */
    public ChargeurTexte() {
        automate = new Automate(this);
    }

    /**
     * Permet de charger un flux depuis l'url url
     * @param url
     * @throws java.io.IOException
     */
    private void chargerFlux(URL url) throws IOException {
        urlConnection = url.openConnection();
        urlConnection.setRequestProperty("User-Agent", "IDXS");
        /*
        String contentType = urlConnection.getContentType();
        if (urlConnection == null || contentType == null ||
        !contentType.contains("text")) {
        //System.out.println("On rejette une url: "+url);
        throw new ChargeurException("L'URL n'est pas du texte : " + url);
        }
         */
        urlConnection.setConnectTimeout(5000);
        urlConnection.setReadTimeout(5000);
        flux = urlConnection.getInputStream();
    }

    /**
     * initialise le flux à partir de l'url url
     * @param url l'url qui doit être traitée
     */
    public void initialiserFlux(URL url) throws ChargeurException {
        charsetTrouve = false;
        //On ouvre un lecteurFlux a partir de l'url du fichier.
        try {  
            chargerFlux(url);
            lecteurFlux = new InputStreamReader(flux);
            automate.reinitialiser();
            urlPageCourante = url;
            charsetTrouve = false;
            if (urlPageCourante.getPath().length() == 0) {
                urlPageCourante = new URL(urlPageCourante.getProtocol(),
                        urlPageCourante.getHost(), "/");
            }
        } catch (IOException ex) {
            if (Util.verbose) {
                ex.printStackTrace();
            }
            throw new ChargeurException(ex.toString());
        }
    }

    /**
     * Methode chargeant le fichier et on envoie ligne par ligne au traiteur
     */
    public void charger() throws ChargeurException {
        //On s'assure qu'un traiteur a ete defini
        if (this.getTraiteur() == null) {
            throw new ChargeurException("Erreur: aucune unit" + '\u00e9' +
                    " de traitement d" + '\u00e9' + "finie!");
        }
        lecteur = new BufferedReader(lecteurFlux);
        try {
            int caractere = lecteur.read();
            while (caractere >= 0 && !charsetTrouve) {
                //on fait traiter le caractère lu par le traiteur
                char car = (char) caractere;
                automate.trouverCharset(car);
                //on lit la ligne suivante
                synchronized (this) {
                    caractere = lecteur.read();
                }
            }
            //on envoie des espaces pour que le tampon de la machine d'état soit
            //consommé
            automate.trouverCharset(' ');
            automate.trouverCharset(' ');
            automate.trouverCharset(' ');
            automate.trouverCharset(' ');
            if (!charsetTrouve) {
                //Aucun encodage défini dans la page
                flux.close();
                chargerFlux(urlPageCourante);
                lecteur.close();
                lecteurFlux = new InputStreamReader(flux);
                lecteur = new BufferedReader(lecteurFlux);
                synchronized (this) {
                    caractere = lecteur.read();
                }
            }
            while (caractere >= 0) {
                //on fait traiter le caractère lue par le traiteur
                char car = (char) caractere;
                automate.instructionEtat(car);

                //on lit la ligne suivante
                synchronized (this) {
                    caractere = lecteur.read();
                }
            }
            //on envoie des espaces pour que le tampon de la machine d'état soit
            //consommé
            automate.instructionEtat(' ');
            automate.instructionEtat(' ');
            automate.instructionEtat(' ');
            automate.instructionEtat(' ');
        } catch (IOException ex) {
            //Si une erreur de lecture/écriture se produit
            if (Util.verbose) {
                ex.printStackTrace();
            }
        } finally {

            try {
                //on ferme le lecteur qui se chargera de fermer le lecteurFlux
                lecteur.close();
                flux.close();
            } catch (IOException ex) {
                if (Util.verbose) {
                    ex.printStackTrace();
                }
            }
        }
    }

    /**
     * Methode permettant à l'automate de signaler le mot qu'il a trouvé
     * @param mot le mot qui a ete trouve
     */
    protected void envoiMot(String mot) {
        this.getTraiteur().traiterMot(mot.toLowerCase());
    }

    /**
     * Methode permettant a l'automate de signaler l'url relative qu'il a trouve
     * @param url la representation sous forme de chaine de l'url trouvee
     */
    protected void envoieURL(String url) {
        try {
            URL a = new URL(urlPageCourante, url);
            //System.out.println("\nurl => "+a.toString());
            this.getTraiteur().traiterURL(a);
        } catch (MalformedURLException ex) {
            if (Util.verbose) {
                System.out.println("URL mal form\u00e9: " + url.toString());
                ex.printStackTrace();
            }
        }
    }

    /**
     * Methode permettant a l'automate de signaler qu'il a trouve le charset
     * de la page
     * @param charset la representation sous forme de chaine du charset
     */
    protected synchronized void changerCharset(String charset) {

        if (!charsetTrouve) {
            //System.out.println("CHARSET TROUVE: "+charset);
            try {
                lecteur.close();
                flux.close();
                chargerFlux(urlPageCourante);
                try {
                    lecteurFlux = new InputStreamReader(flux, charset);
                } catch (UnsupportedEncodingException ex) {
                    lecteurFlux = new InputStreamReader(flux);
                    if (Util.verbose) {
                        System.out.println("Encodage non trouv\u00e9: \"" + charset + "\" dans " +
                                "la page " + urlPageCourante);
                        ex.printStackTrace();
                    }
                }
                lecteur = new BufferedReader(lecteurFlux);
                charsetTrouve = true;
            } catch (IOException ex) {
                if (Util.verbose) {
                    ex.printStackTrace();
                }
            }

        }
    }
}
