package be.ulg.hoessen.martinLeon.crawler;

import be.ulg.hoessen.martinLeon.Util;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import be.ulg.hoessen.martinLeon.indexeur.Index;
import java.util.Collections;
import java.util.Set;

/**
 * Classe permettant de mettre à jour un index en parcourant le web, mais en
 * se limitant aux pages voisines d'une page donnée, restriction sur le domaine
 * et restriction quant à la distance d'une page par rapport à la première page
 * @author benjamin, benoit
 */
public class Crawler {

    /** L'index que l'on mettra à jour                                 */
    private Index index;
    /** La distance maximale que l'on doit chercher                    */
    private int distance;
    /** Le domaine auquel on se restreint                              */
    private String domaine;
    /** La pagesEnAttente des urls déjà parcourues ou qui vont l'être  */
    private Set<String> pages;
    /** Le nombre de robots par défaut                                 */
    private int nbRobots = 10;
    /** permet de connaitre le nombre de robot en cours de travail */
    private volatile int nbRobotsOccupe = 0;
    /** 
     * permet de savoir quel est le robot suivant auquel on doit
     * ajouter une page 
     */
    private int nextRobot = 0;
    /** Permet de savoir si on est en train de parcourir ou non        */
    private boolean enCours;
    /** Retient le nombre de pages que l'on a indexe                   */
    private int nbPages = 0;
    /**
     * La liste des robots en attente. En faisant nbRobots - robots.size()
     * on peut savoir le nombre de robots actifs.
     */
    private volatile ArrayList<Robot> robots;
    /** Liste des robots en attente                                    */
    private ArrayList<Robot> robotsEnAttente;
    /** Permet de retenir quel est la page qui a été donnée en permière*/
    private URLEnAttente premier;
    private int nbLance;

    /**
     * Instancie un nouveau Crawler
     * @param distance la distance maximale depuis la source
     * @param domaine le domaine de restriction
     * @param start la page de depart
     * @param unIndex, l'index que l'on va compléter
     */
    public Crawler(int uneDistance, String unDomaine, URL start, Index unIndex) {
        index = unIndex;
        distance = uneDistance;
        domaine = unDomaine;
        nbLance = 0;
        premier = new URLEnAttente(start, 0, estDansDomaine(start));
        pages = Collections.synchronizedSet(new HashSet<String>());
        robots = new ArrayList<Robot>(nbRobots);
        robotsEnAttente = new ArrayList<Robot>(nbRobots);
        //On crée les différents robots
        for (int i = 0; i < nbRobots; i++) {
            Robot r = new Robot(this, index);
            robots.add(r);
            robotsEnAttente.add(r);
        }

        //pagesEnAttente.add();
        enCours = false;
        index.ajouterDescritpion("URL de d\u00e9part:\n" + start +
                "\nDistance:\n" + distance + "\nDomaine permis:\n\"" + domaine + "\"\n");
    }

    /**
     * Change le nombre de robots
     * @param distance la distance maximale depuis la source
     */
    public void nombreRobots(int nombre) throws CrawlerException {
        if (enCours) {
            throw new CrawlerException("Le parcours du web est d" + '\u00e9' + "j" +
                    '\u00e0' + " en cours");
        } else {
            nbRobots = nombre;
            robots = new ArrayList<Robot>(nbRobots);
            robotsEnAttente = new ArrayList<Robot>(nbRobots);
            //On crée les différents robots
            for (int i = 0; i < nbRobots; i++) {
                Robot r = new Robot(this, index);
                robots.add(r);
                robotsEnAttente.add(r);
            }
        }
    }

    public void pageParcourue(){
        nbPages++;
    }
    
    /**
     * Commence le parcours des différentes pages
     */
    public void start() {
        nbPages = 0;
        enCours = true;

        nbRobotsOccupe++;
        //On affiche le temps passé si on affiche pas les détails
        if (!Util.verbose) {
            Thread t = new Thread(new Runnable() {

                public void run() {
                    afficherInfo();
                }
            });
            t.start();
        }

        //On lance le parcours de la première page
        try {
            Robot r = robots.get(0);
            robotsEnAttente.remove(r);
            r.lancer(premier);
        } catch (RobotException ex) {
            System.out.println(ex.toString());
            ex.printStackTrace();
        }
        /* On boucle tant qu'un robot est occupé */
        while (nbRobotsOccupe > 0) {
        }

        //On boucle pour s'assurer que tous les robots aient bien fini
        while (nbRobotsOccupe > 0) {
            try {
                tuerRobots();
            } catch (RobotException ex) {
                if (Util.verbose) {
                    System.out.println(ex);
                }
                System.out.println("\n\n\n\n\n\n\n\n\n\nAttention, " +
                        "un robot n'avait pas fini lorsqu'on le lui a ordonnée!");
                nbRobotsOccupe++;
                while (nbRobotsOccupe > 0) {
                }
            }
        }

        index.ajouterDescritpion("Nombre de pages visit\u00e9es:\n" + nbPages +
                "\n");
        System.out.println("Nombre de pages visit\u00e9es: " + nbPages);
    }

    private void tuerRobots() throws RobotException {
        for (int i = 0; i < robots.size(); i++) {
            robots.get(i).tuer();
        }
    }

    /**
     * Permet d'afficher en console le temps passé depuis le début
     */
    private void afficherInfo() {
        long debut = System.nanoTime();
        while (nbRobotsOccupe > 0) {
            //System.gc();
            System.out.flush();
            System.out.println("Nombre de robots occup\u00e9:\t" +
                    nbRobotsOccupe);
            System.out.println("Nombre de pages a parcourir:\t" +
                    (nbLance-nbPages));
	    System.out.println("Nombre de pages parcourues:\t"+
			       nbPages);
            double temps = System.nanoTime() - debut;
            System.out.println("temps: " +
                    (int) (temps / 1000000000.0) +
                    " secondes");
            try {
                Thread.sleep(1000);
            } catch (Exception e) {

            }
        }
        double temps = System.nanoTime() - debut;
        System.out.println("temps total: " +
                (int) (temps / 1000000000.0) +
                " secondes");
    }

    /**
     * Pemrmet de tester si il faut parcourir la page ou pas
     * @param url l'url que l'on doit peut-être parcourir
     * @param uneDistance la distance de l'url vis-à-vis de la page initiale
     * @return vrai si l'on doit parcourir la page
     */
    private boolean testerPage(URL url, int uneDistance) {
        boolean estAjoute = false;
        boolean condition = uneDistance <= distance;
        if (condition) {
            estAjoute = pages.add(url.toString());
        }
        return condition && estAjoute;
    }

    /**
     * Ajoute une url dans la pagesEnAttente des urls à parcourir si nécéssaire.
     * @param url l'url que l'on est susceptible de parcourir.
     * @param distance la distance par rapport à la première page que l'on
     * à parcouru
     */
    public void parcourirUrl(URL url, int uneDistance, boolean dansDomaine) {
        boolean condition = testerPage(url, uneDistance);
        boolean pasCalendrier = !(url.toString().startsWith("http://www.montefiore.ulg.ac.be/~maree/calendar-conferences") || url.toString().startsWith("http://www.montefiore.ulg.ac.be/setlanguage.php"));
        if(!pasCalendrier && Util.verbose){
            System.out.println("url rejetée par le filtre:");
            System.out.println(url);
        }
        condition = condition && pasCalendrier;
        if (condition) {
            boolean traiter = estDansDomaine(url);
            if ((traiter && !dansDomaine) || dansDomaine) {
                URLEnAttente urlEnAttente =
                        new URLEnAttente(url, uneDistance, traiter);
                Robot tmp = null;
                if (nbRobotsOccupe < nbRobots) {
                    synchronized (robotsEnAttente) {
                        if (robotsEnAttente.size() > 0) {
                            tmp = robotsEnAttente.remove(robotsEnAttente.size() - 1);
                            nbRobotsOccupe++;
                        }
                    }
                }
                if (tmp == null) {
                    tmp = robots.get(nextRobot);
                    nextRobot = (nextRobot + 1) % nbRobots;
                }
                try {
                    tmp.lancer(urlEnAttente);
                    nbLance++;
                } catch (RobotException ex) {
                    System.out.println(ex.toString());
                    ex.printStackTrace();
                }
            }
        }
    }

    /**
     * Permet de signaler au crawler qu'un robot à fini son travail
     * @param a le robot qui vient de finir son travail
     */
    public void signalFini(Robot a) {
        synchronized (robotsEnAttente) {
            robotsEnAttente.add(a);
            nbRobotsOccupe--;
        }

    }

    /**
     * Teste si la page est dans le domaine voulu
     * @param url l'url que l'on veut tester
     * @return true si l'url est dans le domaine
     */
    private boolean estDansDomaine(URL url) {
        boolean tmp = url.getHost().toString().endsWith(domaine);
        return tmp ;
    }
}
