package be.ulg.hoessen.martinLeon.indexeur;

import be.ulg.hoessen.martinLeon.Util;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;
import javax.swing.filechooser.FileSystemView;

/**
 * Classe représentant l'index par le biais d'une HashMap.
 * @author benoit, benjamin
 */
public class Index implements Serializable {

    private Map<String, IndexMot> index;
    private String description;
    private final static long serialVersionUID = 20071211;

    public Index() {
        description = "";
        index = Collections.synchronizedMap(new HashMap<String, IndexMot>());
    }

    /**
     * permet d'ajouter un mot (trouvé dans une page web d'adresse "source")
     * dans l'index
     * @param source l'adresse de la page qui contient le mot
     * @param mot le mot que l'on veut ajouter dans l'index
     */
    public void add(URL source, String mot) {
        IndexMot hVal = index.get(mot);
        // cas où le mot est déjà présent dans la HashMap
        synchronized (index) {
            if (hVal != null) {
                hVal.incNbOccTot();
                hVal.addPgWeb(source);
            } // cas ou le mot n'est pas déjà présent dans la Hashmap
            else {
                hVal = new IndexMot();
                hVal.addPgWeb(source);
                index.put(mot, hVal);
            }
        }
    }

    /**
     * Permet d'ajouter une description à l'index
     * @param d une description
     */
    public synchronized void ajouterDescritpion(String d) {
        description += d;
    }

    public IndexMot listeUrlContenant(String mot) {
        IndexMot tmp = index.get(mot);
        if(tmp==null){
            tmp = new IndexMot();
        }
        return tmp;
    }

    public ArrayList<URL> resultat(String expression) throws IndexException {
        IndexMot tmp = Noeud.resultat(this, expression);
        if (tmp != null) {
            return tmp.getListe();
        } else {
            throw new IndexException("Le mot demand\u00e9 n'est pas dans l'index");
        }
    }

    /**
     * Renvoie un String représentant l'index
     * @return la représentation sous forme de String de l'index
     */
    @Override
    public synchronized String toString() {
        //On crée le stringBuffer avec une approximation de la taille finale
        StringBuffer chaine = new StringBuffer(index.size() * 1000);
        chaine.append("Index :\n");
        chaine.append("-------\nNombre de mots: ");
        chaine.append(index.size());
        chaine.append('\n');
        Iterator clefs = index.keySet().iterator();
        String separateur = ": ";
        for (Iterator i = index.values().iterator(); i.hasNext();) {
            chaine.append(clefs.next().toString());
            chaine.append(separateur);
            chaine.append(i.next().toString());
            chaine.append('\n');
        }
        return chaine.toString();
    }

    /**
     * Renvoie le nombre de mots contenus dans l'index
     * @return le nombre de mots dans l'index
     */
    public int nbMots() {
        return index.size();
    }

    /**
     * Permet d'enregistrer un index
     * @param fichier là ou on doit enregistrer le fichier
     */
    public void enregistrerIndex(File fichier) throws FileNotFoundException, IOException {
        File tmpDesc = null;
        File tmpSer = null;
        if (fichier.getParentFile().isDirectory()) {
            tmpDesc = new File(fichier.getParentFile(), "tmpDesc.tdx");
            tmpSer = new File(fichier.getParentFile(), "tmpSer.tdx");
        } else {
            tmpDesc = new File(FileSystemView.getFileSystemView().
                    getDefaultDirectory(), "tmpDesc.tdx");
            tmpSer = new File(FileSystemView.getFileSystemView().
                    getDefaultDirectory(), "tmpSer.tdx");
        }

        //on crée l'archive qui contiendra la description et le fichier
        int BUFFER = 2048;
        byte data[] = new byte[BUFFER];
        FileOutputStream dest = new FileOutputStream(fichier);
        ZipOutputStream zipDest = new ZipOutputStream(dest);
        zipDest.setMethod(ZipOutputStream.DEFLATED);
        zipDest.setLevel(9);

        //on enregistre la description
        FileWriter fWriter = new FileWriter(tmpDesc);
        BufferedWriter tampon = new BufferedWriter(fWriter);
        PrintWriter sortie = new PrintWriter(tampon);
        sortie.println(description);
        sortie.close();
        //on ajoute la description
        FileInputStream fi = new FileInputStream(tmpDesc);
        BufferedInputStream buffi = new BufferedInputStream(fi, BUFFER);
        ZipEntry entry = new ZipEntry("desc.didx");
        zipDest.putNextEntry(entry);
        int count;
        while ((count = buffi.read(data, 0, BUFFER)) != -1) {
            zipDest.write(data, 0, count);
        }
        zipDest.closeEntry();
        //on supprime le fichier temporaire
        tmpDesc.delete();

        //on enregistre l'index
        ObjectOutputStream out =
                new ObjectOutputStream(new FileOutputStream(tmpSer));
        out.writeObject(this);
        out.close();
        //on ajoute l'index
        fi = new FileInputStream(tmpSer);
        buffi = new BufferedInputStream(fi, BUFFER);
        entry = new ZipEntry("serial.sidx");
        zipDest.putNextEntry(entry);
        count = 0;
        while ((count = buffi.read(data, 0, BUFFER)) != -1) {
            zipDest.write(data, 0, count);
        }
        zipDest.closeEntry();
        //on supprime le fichier temporaire
        tmpSer.delete();

        //on ferme l'archive
        zipDest.close();

    }

    /**
     * Permet d'obtenir la description d'un index enregistré sur le disque
     * @param fichier l'index sur le disque
     * @return une description de l'index 
     */
    public static String chargerDescription(File fichier) throws IOException {
        ZipFile zip = new ZipFile(fichier);
        InputStream flux = zip.getInputStream(zip.getEntry("desc.didx"));
        BufferedReader lecteur = new BufferedReader(new InputStreamReader(flux));
        StringBuffer description = new StringBuffer();
        String tmp = lecteur.readLine();
        while (tmp != null) {
            description.append(tmp);
            description.append('\n');
            tmp = lecteur.readLine();
        }
        zip.close();
        lecteur.close();
        return description.toString();
    }

    /**
     * Permet de charger un index enregistré sur un disque
     * @param fichier le fichier qui contient l'index
     * @return l'index
     */
    public static Index chargerIndex(File fichier) throws
            ZipException, IOException, ClassNotFoundException {
        Index index = null;
        ZipFile zip = new ZipFile(fichier);
        InputStream flux = zip.getInputStream(zip.getEntry("serial.sidx"));
        ObjectInputStream in = new ObjectInputStream(flux);
        Object a = in.readObject();
        in.close();
        zip.close();
        if (a instanceof Index) {
            index = (Index) a;
            Util.message("Chargement de l'index réussi.");
            return index;
        } else {
            throw new ClassNotFoundException(
                    "Le fichier fourni n'est pas un index");
        }


    }
}
