package be.ulg.hoessen.martinLeon.indexeur;

import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;

/**
 * Classe représentant la valeur de la HashMap
 * composée du nombre d'occurences total et
 * d'un IndexURL qui est la racine de l'arbre.
 *
 * @author benoit, benjamin
 */
public class IndexMot implements Serializable {

    /** Le nombre total d'occurrences du mot */
    private int nbOccTot;
    /** racine de l'arbre */
    private IndexURL url;
    private final static long serialVersionUID = 20071130;

    /**
     * Instancie une nouvelle IndexMot
     */
    public IndexMot() {
        // mis à 1 directement pour éviter de devoir utiliser
        // la méthode incNbOccTot() à chaque instanciation de
        // la classe IndexMot
        nbOccTot = 1;
    }

    /**
     * Instancie un nouvel IndexMot
     * @param place la place que l'on prévoit déjà pour le nombre d'url
     */
    public IndexMot(int place) {
        nbOccTot = 1;
    }

    /**
     * renvoit le nombre total d'occurences du mot
     * @return nbOccTot
     */
    public int getNbOccTot() {
        return nbOccTot;
    }

    /**
     * incrémente le nombre total d'occurences du
     * mot
     */
    public void incNbOccTot() {
        nbOccTot++;
    }

    /**
     * ajoute une url à l'arbre
     * @param adresse l'adresse qui contient le mot
     */
    public void addPgWeb(URL adresse) {
        // cas où l'IndexMot est vide donc création de la racine
        if (url == null) {
            url = new IndexURL(adresse);
        } else {
            url.ajouter(adresse);
        }
    }

    /**
     * Renvoie un String représentant l'index
     * 
     * @return la représentation sous forme de String de l'index
     */
    @Override
    public String toString() {
        StringBuffer chaine = new StringBuffer();
        chaine.append(nbOccTot);
        chaine.append(" occurrence(s) dans ");
        chaine.append(url.getNbPgWeb());
        chaine.append(" page(s)");
        chaine.append("\nadresses des pages:\n");
        ArrayList<URL> liste = getListe();
        if (liste == null) {
            System.out.println("la liste est null");
        }
        for (int i = 0; i < liste.size(); i++) {
            chaine.append(liste.get(i));
        }
        return chaine.toString();
    }

    /**
     * renvoit une ArrayList qui contient les adresses contenant
     * le mot
     * @return une ArrayList contenant les adresses qui contiennent
     * le mot
     */
    public ArrayList<URL> getListe() {
        if (url != null) {
            ArrayList<IndexURL> tmp = new ArrayList<IndexURL>(url.getNbPgWeb());
            //on met tous les éléments dans une liste
            url.putAll(tmp);
            //on trie la liste
            if (tmp.size() > 0) {
                tmp = quicksort(tmp, 0, tmp.size() - 1);
            }
            ArrayList<URL> list = new ArrayList<URL>(tmp.size());
            for (int i = 0; i < tmp.size(); i++) {
                list.add(tmp.get(i).getUrl());
            }
            return list;
        } else {
            return new ArrayList<URL>();
        }

    }

    /**
     * permet de faire un quicksort
     */
    private static ArrayList<IndexURL> quicksort(ArrayList<IndexURL> tableau,
            int gauche,
            int droite) {
        int g, d;
        IndexURL pivot, tmp;
        g = gauche;
        d = droite + 1;
        pivot = tableau.get(gauche);
        do {
            do {
                g++;
            } while (g <= droite && tableau.get(g).compareTo(pivot) < 0);
            do {
                d--;
            } while (tableau.get(d).compareTo(pivot) > 0);

            if (g < d) {
                tmp = tableau.get(g);
                tableau.set(g, tableau.get(d));
                tableau.set(d, tmp);
            }
        } while (g <= d);
        tmp = tableau.get(gauche);
        tableau.set(gauche, tableau.get(d));
        tableau.set(d, tmp);
        if (gauche < d) {
            quicksort(tableau, gauche, d);
        }
        if (g < droite) {
            quicksort(tableau, g, droite);
        }
        return tableau;
    }

    /**
     * Permet d'avoir l'intersection entre la liste des urls de cette instance et
     * celle contenue dans autre. Une url sera contenu dans l'IndexMot renvoyé
     * si et seulement si il est contenu dans les deux listes
     * @param autre l'autre IndexMot avec lequel on veut avoir l'intersection.
     * @return un IndexMot qui contient l'intersection des deux instances.
     */
    public IndexMot intersection(IndexMot autre) {
        IndexMot indM = new IndexMot();
        indM.url = this.url.intersection(autre.url, null);
        return indM;
    }

    /**
     * Permet de faire l'union de deux ensembles
     */
    public IndexMot union(IndexMot autre) {
        IndexMot indM = new IndexMot();
        indM.url = this.url.union(autre.url);
        return indM;
    }
}
