package be.ulg.hoessen.martinLeon.indexeur;

import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;

/**
 * Classe représentant un noeud de l'arbre
 * composée d'un entier qui représente le nombre
 * d'occurrences du mot dans la page inUrl ainsi 
 * que du nombre de noeuds dans l'arbre (size)
 * et de deux champs pour les deux fils respectifs 
 * du noeud.
 * @author benoit, benjamin
 */
public class IndexURL implements Comparable, Serializable {

    private int nbOccMot;
    private URL inUrl;
    private static int size;
    private final static long serialVersionUID = 20071119;
    private IndexURL filsG,  filsD;

    public IndexURL(URL u) {
        // mis à 1 directement pour éviter de devoir utiliser
        // la méthode incNbOccMot() à chaque instanciation de
        // la classe IndexURL
        nbOccMot = 1;
        inUrl = u;
        filsG = null;
        filsD = null;
    }

    /**
     * renvoit le fils à gauche du noeud
     *
     */
    public IndexURL getFilsG() {
        return filsG;
    }

    /**
     * renvoit le fils à droite du noeud
     *
     */
    public IndexURL getFilsD() {
        return filsD;
    }

    /**
     * renvoit le nombre de fois que le mot est contenu
     * dans la page
     * @return le nombre d'occurrences du mot dans la page
     */
    public int getNbOccMot() {
        return nbOccMot;
    }

    /**
     * incrémente de 1 le nombre de fois que le mot est
     * contenu dans la page
     *
     */
    public void incNbOccMot() {
        nbOccMot++;
    }

    /**
     * renvoit l'url liée à cet objet
     * @return l'url
     */
    public URL getUrl() {
        return inUrl;
    }

    /**
     * méthode de comparaison
     * @return un entier négatif, 0 ou un entier positif
     * si cet objet est plus petit, =, ou plus grand que
     * l'autre objet
     */
    public int compareTo(Object autre) {
        if (autre instanceof IndexURL) {
            return ((IndexURL) autre).nbOccMot - nbOccMot;
        } else {
            return -1;
        }
    }

    public int hashCode() {
        return inUrl.hashCode();
    }

    /**
     * permet d'ajouter une url à la bonne place dans l'arbre
     * @param noeud: l'IndexURL à ajouter
     * @return l'IndexURL ajouté
     */
    public void ajouter(URL adresse) {

        String chaine = adresse.toString();
        IndexURL tmp = this;
        boolean stop = false;
        String chaine2;

        while (!stop) {
            chaine2 = tmp.inUrl.toString();
            if (chaine.compareTo(chaine2) == 0) {
                tmp.incNbOccMot();
                stop = true;
            } else if (chaine.compareTo(chaine2) < 0) {
                if (tmp.filsG == null) {
                    tmp.filsG = new IndexURL(adresse);
                    IndexURL.size++;
                    stop = true;
                } else {
                    tmp = tmp.filsG;
                }
            } else if (chaine.compareTo(chaine2) > 0) {
                if (tmp.filsD == null) {
                    tmp.filsD = new IndexURL(adresse);
                    IndexURL.size++;
                    stop = true;
                } else {
                    tmp = tmp.filsD;
                }
            }
        }

    }

    // renvoit la taille de l'arbre
    public int getNbPgWeb() {
        return size;
    }

    /**
     * permet de mettre tous les elements de l'arbre dans une liste
     *
     */
    public void putAll(ArrayList<IndexURL> list) {
        if (filsG != null) {
            filsG.putAll(list);
        }

        list.add(this);

        if (filsD != null) {
            filsD.putAll(list);
        }
    }

    // compare si l'arbre sur lequel est appelé cette methode contient l'url passée en param
    private IndexURL estPresent(URL ind) {
        String chaine = ind.toString();
        String chaine2;

        IndexURL tmp = this;

        while (true) {
            chaine2 = tmp.inUrl.toString();

            if (chaine2.compareTo(chaine) == 0) {
                return tmp;
            } else if (chaine2.compareTo(chaine) < 0) {
                if (tmp.filsD == null) {
                    return null;
                } else {
                    tmp = tmp.filsD;
                }
            } else if (chaine2.compareTo(chaine) > 0) {
                if (tmp.filsG == null) {
                    return null;
                } else {
                    tmp = tmp.filsG;
                }
            }

        }

    }

    /**
     * intersection entre deux arbres
     *
     */
    public IndexURL intersection(IndexURL autre, IndexURL resultat) {
        IndexURL ind = autre.estPresent(this.inUrl);
        if (ind != null) {
            IndexURL nouveau = new IndexURL(this.inUrl);
            nouveau.nbOccMot = Math.min(ind.nbOccMot, this.nbOccMot);
            if (resultat == null) {
                resultat = nouveau;
            } else {
                resultat.ajouterNoeud(nouveau);
            }
        }
        if (filsD != null) {
            this.filsD.intersection(autre, resultat);
        }
        if (filsG != null) {
            this.filsG.intersection(autre, resultat);
        }
        return resultat;
    }

    /**
     * méthode privée de copie
     *
     */
    private void copie(IndexURL resultat) {
        IndexURL tmp = new IndexURL(this.inUrl);
        tmp.nbOccMot = this.nbOccMot;
        resultat.ajouterNoeud(tmp);
        if (filsD != null) {
            filsD.copie(resultat);
        }
        if (filsG != null) {
            filsG.copie(resultat);
        }
    }

    /**
     * union entre deux arbres
     *
     */
    public IndexURL union(IndexURL autre) {
        IndexURL resultat = new IndexURL(this.inUrl);
        resultat.nbOccMot = this.nbOccMot;
        if (filsD != null) {
            filsD.copie(resultat);
        }
        if (filsG != null) {
            filsG.copie(resultat);
        }
        autre.copie(resultat);
        return resultat;
    }

    /**
     * permet d'ajouter un noeud dans l'arbre
     *
     */
    public void ajouterNoeud(IndexURL noeud) {

        String chaine = noeud.inUrl.toString();
        IndexURL tmp = this;
        boolean stop = false;

        while (!stop) {
            if (chaine.compareTo(tmp.inUrl.toString()) < 0) {
                if (tmp.filsG == null) {
                    tmp.filsG = noeud;
                    IndexURL.size++;
                    stop = true;
                } else {
                    tmp = tmp.filsG;
                }
            } else if (chaine.compareTo(tmp.inUrl.toString()) > 0) {
                if (tmp.filsD == null) {
                    tmp.filsD = noeud;
                    IndexURL.size++;
                    stop = true;
                } else {
                    tmp = tmp.filsD;
                }
            } else {
                tmp.nbOccMot += noeud.nbOccMot;
                stop = true;
            }
        }
    }
}
