package be.ulg.hoessen.martinLeon.indexeur;

/**
 * classe qui définit un noeud de l'arbre
 * @author benoît, benjamin
 *
 */
public class Noeud {

    private IndexMot info;
    private Noeud frere;
    private Noeud fils;
    /** add vaut true si le noeud est un opérateur + ;
    add vaut false si le noeud est un opérateur * ;
    si info est différent de null alors on ne s'occupe pas de
    la valeur de add	
     */
    private boolean add;

    /**
     * constructeur de noeud
     * @param info, l'IndexMot associé au noeud ou null si
     * 		  le noeud est un opérateur
     * @param add, dans le cas où info est null, détermine 
     * 		  l'opérateur (+ ou *)
     */
    public Noeud(IndexMot info, boolean add) {
        frere = null;
        fils = null;
        this.info = info;
        this.add = add;
    }

    /**
     * méthode qui permet d'ajouter un frère à un noeud
     * @param noeud, le noeud à ajouter comme frère
     * @return le noeud ajouté
     */
    public Noeud ajouterFrere(Noeud noeud) {
        // cas où le noeud n'a pas de frère
        if (frere == null) {
            frere = noeud;
        // cas où le noeud a déja au moins un frère
        } else {
            Noeud tmp = frere;
            frere = noeud;
            noeud.frere = tmp;
        }
        return noeud;
    }

    /**
     * méthode qui permet d'ajouter un fils à un noeud
     * @param noeud, le noeud à ajouter comme fils
     * @return le noeud ajouté
     */
    public Noeud ajouterFils(Noeud noeud) {
        // cas où le noeud n'a pas de fils
        if (fils == null) {
            fils = noeud;
        // cas où le noeud a déja au moins un fils
        } else {
            fils.ajouterFrere(noeud);
        }
        return noeud;
    }

    /**
     * méthode qui étant donné un arbre, calcule le résultat de celui-ci
     * @param plus, true si le noeud a un opérateur + comme père, 
     * 		  false si c'est un *,
     * 		  pour le premier appel la valeur de plus 
     * 		  n'a pas d'importance (non utilisée)  
     * @return l'IndexMot calculé
     */
    public IndexMot calculerResultat(boolean plus) {
        // IndexMot renvoyé
        IndexMot postResultat;
        // cas d'un mot
        if (info != null) {
            // cas où le noeud a un frère
            if (frere != null) {
                // + comme père
                if (plus) {
                    postResultat = info.union(frere.calculerResultat(plus));
                // * comme père
                } else {
                    postResultat = info.intersection(
                            frere.calculerResultat(plus));
                }
            // cas où le noeud n'a pas de frère
            } else {
                postResultat = info;
            }
        // cas d'un opérateur
        } else {
            // +
            if (add) {
                postResultat = fils.calculerResultat(true);
            // *
            } else {
                postResultat = fils.calculerResultat(false);
            }
            // cas où le noeud a un frère
            if (frere != null) {
                // + comme pere
                if (plus) {
                    postResultat = postResultat.union(
                            frere.calculerResultat(plus));
                // * comme père
                } else {
                    postResultat = postResultat.intersection(
                            frere.calculerResultat(plus));
                }
            }
        }
        return postResultat;
    }

    /**
     * classe statique privée qui permet de mémoriser un noeud et un indice 
     * @author benoît, benjamin
     *
     */
    private static class ParseInfo {
        // indice qui permet de mémoriser où on se trouve dans la ligne parsée
        int i;
        // noeud à renvoyer
        Noeud n;
    }

    /**
     * méthode qui permet de parser la ligne de caractères entrée comme
     * recherche par l'utilisateur (construction de l'arbre)
     * @param index, sert à récuperer les données nécessaires 
     * @param ligne, la ligne à parser
     * @param i, l'indice où on en est arrivé dans la ligne
     * @return le noeud et l'indice adéquats (instance de ParseInfo) 
     */
    private static ParseInfo parser(Index index, String ligne, int i) {
        Noeud racine = null;
        ParseInfo pInfo = new ParseInfo();
        char tmp;
        StringBuffer tampon = new StringBuffer();
        int lg = ligne.length();
        while (i < lg) {
            tmp = ligne.charAt(i);
            // cas d'un espace vide
            if (tmp == ' ') {
                if (tampon.length() > 0) {
                    /* construction et placement dans l'arbre du noeud associé
                    au mot présent dans le tampon */
                    racine.ajouterFils(new Noeud(
                            index.listeUrlContenant(tampon.toString()), true));
                    tampon.delete(0, tampon.length());
                }
            // cas d'un +
            } else if (tmp == '+') {
                racine = new Noeud(null, true);
            // cas d'un *
            } else if (tmp == '*') {
                racine = new Noeud(null, false);
            // cas d'une parenthèse ouvrante
            } else if (tmp == '(') {
                if (tampon.length() > 0) {
                    /* construction et placement dans l'arbre du noeud associé
                    au mot présent dans le tampon */
                    racine.ajouterFils(new Noeud(
                            index.listeUrlContenant(tampon.toString()), true));
                    tampon.delete(0, tampon.length());
                }
                // récursion
                pInfo = Noeud.parser(index, ligne, i + 1);
                // mise à jour de l'indice
                i = pInfo.i;
                // ajout du noeud comme fils
                racine.ajouterFils(pInfo.n);
            // cas d'une parenthèse fermante
            } else if (tmp == ')') {
                if (tampon.length() > 0) {
                    /* construction et placement dans l'arbre du noeud associé
                    au mot présent dans le tampon */
                    racine.ajouterFils(new Noeud(
                            index.listeUrlContenant(tampon.toString()), true));
                    tampon.delete(0, tampon.length());
                }
                // attribution des valeurs à pInfo  
                pInfo.i = i;
                pInfo.n = racine;
                return pInfo;
            // cas d'un autre caractère  
            } else {
                tampon.append(tmp);
            }
            i++;
        }
        if (tampon.length() > 0) {
            /* construction et placement dans l'arbre du noeud associé
            au mot présent dans le tampon */
            racine.ajouterFils(new Noeud(
                    index.listeUrlContenant(tampon.toString()), true));
            tampon.delete(0, tampon.length());
        }
        // attribution des valeurs à pInfo  
        pInfo.i = i;
        pInfo.n = racine;
        return pInfo;
    }

    /**
     * méthode qui permet d'obtenir le résultat de la requête 
     * "expression" sur l'index
     * @param index, sert à récupérer les données nécessaires
     * @param expression, l'expression à traîter
     * @return le résultat (un IndexMot)
     */
    public static IndexMot resultat(Index index, String expression) {
        if (expression.contains("+".subSequence(0, 1)) ||
                expression.contains("*".subSequence(0, 1))) {
            return parser(index, expression, 0).n.calculerResultat(true);
        // cas où il y a uniquement des mots en entrée
        } else {
            expression = '*' + expression;
            return parser(index, expression, 0).n.calculerResultat(true);
        }
    }
}
