package be.ulg.hoessen.martinLeon.interfaceGraphique;

import be.ulg.hoessen.martinLeon.crawler.Crawler;
import be.ulg.hoessen.martinLeon.indexeur.Index;
import be.ulg.hoessen.martinLeon.Util;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
/**
 * Classe permettant de créer un index à l'aide d'une interface graphique
 * @author benjamin, benoit
 */
public class CreerIndex extends JFrame implements Runnable {
    
       
    /**
     * Crée une nouvelle fenêtre
     * @param p les preferences car on en aura besoin pour le nombre de robot
     * que l'on utilisera
     * @param parent le composant parent par rapport auquel on doit se centrer
     */
    public CreerIndex(Index idx, Preferences p, Component parent, String titre){
        composentParent = parent;
        prefs = p;
        index = idx;
        initialiserComposants();
        setTitle(titre);
        setVisible(true);
    }
    
    /**
     * Permet d'initaliser les composants
     */
    private void initialiserComposants() {
        //on indique la mise en page de la fenêtre
        getContentPane().setLayout(new BorderLayout());
        
        //On crée le panneau central
        panneauCentral = new JPanel();
        panneauCentral.setLayout(new GridLayout(4, 2, 4, 2));
        getContentPane().add(panneauCentral,BorderLayout.CENTER);
        
        //on ajoute un label qui indique ce qu'on attends dans le champs texte
        labelAdresse = new JLabel();
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        labelAdresse.setText("Adresse de d\u00e9part");
        panneauCentral.add(labelAdresse);
        
        //on ajoute un champ texte pour récupérer l'adresse de départ
        adresse = new JTextField();
        adresse.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent evt) {
                if(evt.getKeyCode() == KeyEvent.VK_ENTER){
                    lancerCreation();
                }
            }
        });
        adresse.setText("http://");
        panneauCentral.add(adresse);
        
        //on ajoute un label qui indique ce qu'on attends dans le champs texte
        labelHorizon = new JLabel();
        labelHorizon.setText("Horizon  ( profondeur )  ");
        panneauCentral.add(labelHorizon);
        
        //on ajoute un champ texte pour récupérer l'horizon
        horizon = new JTextField();
        horizon.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent evt) {
                if(evt.getKeyCode() == KeyEvent.VK_ENTER){
                    lancerCreation();
                }
            }
        });
        horizon.setText("0");
        panneauCentral.add(horizon);
        
        //on ajoute un label qui indique ce qu'on attends dans le champs texte
        labelDomaine = new JLabel();
        labelDomaine.setText("Domaine");
        panneauCentral.add(labelDomaine);
        
        //on ajoute un champ texte pour récupérer le domaine
        domaine = new JTextField();
        domaine.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent evt) {
                if(evt.getKeyCode() == KeyEvent.VK_ENTER){
                    lancerCreation();
                }
            }
        });
        domaine.setText("");
        panneauCentral.add(domaine);
        
        
        //on crée le panneau qui contiendra les boutons
        JPanel panneauBas = new JPanel();
        panneauBas.setLayout(new GridLayout(1,3));
        
        //on crée le bouton qui permet de valider un choix
        boutonOk = new JButton();
        boutonOk.setText("OK");
        boutonOk.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent evt) {
                lancerCreation();
            }
        });
        boutonOk.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent evt) {
                if(evt.getKeyCode() == KeyEvent.VK_ENTER){
                    lancerCreation();
                }
            }
        });
        panneauBas.add(boutonOk);
        
        //On met de l'espace vide entre les deux boutons
        panneauBas.add(new JPanel());
        
        //on crée le bouton pour annuler
        boutonAnnuler = new JButton();
        boutonAnnuler.setText("ANNULER");
        boutonAnnuler.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent evt) {
                annuler();
            }
        });
        boutonAnnuler.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent evt) {
                if(evt.getKeyCode() == KeyEvent.VK_ENTER){
                    annuler();
                }
            }
        });
        panneauBas.add(boutonAnnuler);
        
        //on ajoute le panneau bas dans la fenêtre
        JPanel tmp = new JPanel();
        tmp.add(panneauBas);
        getContentPane().add(tmp,BorderLayout.SOUTH);
        
        //on ajoute de l'espace autour des composants
        getContentPane().add(new JPanel(),BorderLayout.EAST);
        getContentPane().add(new JPanel(),BorderLayout.WEST);
        getContentPane().add(new JPanel(),BorderLayout.NORTH);
        
        //on valide les insertions dans la fenêtre
        pack();
        
        //on spécifie quelques paramètre de la fenêtre
        setResizable(false);
        
        setLocationRelativeTo(composentParent);
        
    }
    
    /**
     * Annule les entrées et quitte la fenêtre
     */
    private void annuler() {
        dispose();
    }
    
    /**
     * lance la création de l'index dans un processus dédié
     */
    private void lancerCreation() {
        try {
            //on va chercher les valeurs entrées
            int distance = Integer.parseInt(horizon.getText());
            if(distance<0){
                Util.erreur("Erreur: l'horizon est n\u00e9gatif, l'horizon " +
                        "par d\u00e9faut est s\u00e9lectionn\u00e9: 0.");
                distance = 0;
            }
            String dom = domaine.getText();
            URL url = new URL(adresse.getText());
            
            //on initialise le crawler
            crawler = new Crawler(distance, dom, url, index);
            
            //on récupère le nombre de robots que l'utilisateur veut
            Object tmp = prefs.valeur(Preferences.NB_ROBOTS);
            if(tmp != Preferences.DEFAULT){
                try{
                    crawler.nombreRobots(Integer.parseInt(tmp.toString()));
                }catch(Exception e){
                    //on n'a rien à faire, 
                    //car crawler à déjà une valeur par défaut'
                }
            }
            
            //on instancie un nouveau processus
            Thread t = new Thread(this);
            t.start();
            
            //on prévient l'utilisateur de ce qui va se passer
            JOptionPane.showMessageDialog(null,"Un pop-up vous pr\u00e9viendra"+
                    " lorsque l'index aura \u00e9t\u00e9 compl\u00e8tement " +
                    "cr\u00e9e. \nVous pouvez faire des requ\u00eates sur l'" +
                    "index en construction");
            //on quitte et détruit la fenêtre
            dispose();
        } catch (MalformedURLException ex) {
            ex.printStackTrace();
            Util.erreur(ex.toString());
            dispose();
        }
    }
    
    /**
     * Méthode nécéssaire pour l'exécution du crawler dans un processus dédié
     */
    public void run(){
        if(crawler != null){
            long debut = System.nanoTime();
            //On lance le crawler
            crawler.start();
            double temps = System.nanoTime()-debut;
            //on avertit l'utilisateur que l'on a fini, et on lui indique 
            //le temps qui fut nécéssaire
            JOptionPane.showMessageDialog(null,"La construction de l'index " +
                    "est finie.\n<html><u>Temps n\u00e9c\u00e9ssaire:</u>" +
                    "</html>\n    " + temps/1000000000.0+ " secondes. \n" +
                    "<html><u>Nombre de mots trouv\u00e9s:</u></html>" +
                    "\n    " + index.nbMots());
            crawler = null;
            System.gc();
        }
    }
    
    
    private Component composentParent;
    private JButton boutonOk;
    private JButton boutonAnnuler;
    private JLabel labelAdresse;
    private JLabel labelHorizon;
    private JLabel labelDomaine;
    private JTextField adresse;
    private JTextField horizon;
    private JTextField domaine;
    private JPanel panneauCentral;
    private Index index;
    private Crawler crawler;
    private Preferences prefs;
    private final static long serialVersionUID = 20071115;
    
    
}