package be.ulg.hoessen.martinLeon.interfaceGraphique;

import be.ulg.hoessen.martinLeon.indexeur.Index;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

/**
 * Cette classe est une fenêtre qui est utilisée pour regrouper les différents
 * éléments graphique pour la construction et l'utilisation d'un index
 * @author benjamin, benoit
 */
public class Fenetre extends JFrame {
    
    /**
     * Crée une nouvelle fenêtre
     */
    public Fenetre() {
        //on crée l'index
        index = new Index();
        //on charge les préférences
        prefs = Preferences.charger();
        initialiserComposants();
        this.setTitle("IDXS");
        this.setSize(800,600);
        this.setResizable(false);
        this.setVisible(true);
        selecteur = new SelecteurFichier(this);
    }
    
    /**
     * On initialise les différents composants de la fenêtre
     */
    private void initialiserComposants() {
        
        
        //On crée une barre de menu
        barreMenu = new JMenuBar();
        setJMenuBar(barreMenu);
        
        //--------------------------------------------------------------
        
        //On crée un menu "Fichier"
        menuFichier = new JMenu();
        menuFichier.setText("Fichier");
        menuFichier.setMnemonic(KeyEvent.VK_F10);
        barreMenu.add(menuFichier);
        
        //On crée un item "Nouvel Index"
        menuNouveau = new JMenuItem();
        menuNouveau.setText("Cr\u00e9er un nouvel l'index");
        menuNouveau.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                nouvelIndex();
            }
        });
        menuNouveau.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        menuFichier.add(menuNouveau);
        
        //On crée un item "Agrandir index"
        menuAgrandir = new JMenuItem();
        menuAgrandir.setText("Agrandir l'index");
        menuAgrandir.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                agrandirIndex();
            }
        });
        menuAgrandir.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_G, ActionEvent.CTRL_MASK));
        menuFichier.add(menuAgrandir);
        
        /*JMenuItem t = new JMenuItem();
        t.setText("Enregistrer la liste des mots en mode texte");
        t.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                JFileChooser chooser = new JFileChooser();
                chooser.showSaveDialog(null);
                File fichier = chooser.getSelectedFile();
                PrintStream sortie;
                if(fichier!=null){
                    try {
                        sortie = new PrintStream(fichier);
                    } catch (IOException ex) {
                        sortie = System.out;
                    }
                } else {
                    sortie = System.out;
                }
                sortie.print(index.toString());
            }
        });
        t.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_I, ActionEvent.CTRL_MASK));
        menuFichier.add(t);
         **/
        
        //on crée un item "Ouvrir un index"
        menuOuvrir = new JMenuItem();
        menuOuvrir.setText("Ouvrir un index");
        menuOuvrir.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                ouvrirIndex();
            }
        });
        menuOuvrir.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        menuFichier.add(menuOuvrir);
        
        //on crée un item "Enregistrer"
        menuEnregistrer = new JMenuItem();
        menuEnregistrer.setText("Enregistrer l'index");
        menuEnregistrer.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                enregistrerIndex();
            }
        });
        menuEnregistrer.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        menuFichier.add(menuEnregistrer);
        
        
        
        //on crée un item "Préférences"
        menuPreferences = new JMenuItem();
        menuPreferences.setText("Pr\u00e9f\u00e9rences");
        menuPreferences.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                preferences();
            }
        });
        menuPreferences.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_P, ActionEvent.CTRL_MASK));
        menuFichier.add(menuPreferences);
        
        //on crée un item "Quitter"
        menuQuitter = new JMenuItem();
        menuQuitter.setText("Quitter");
        menuQuitter.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                quitter();
            }
        });
        menuQuitter.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
        menuFichier.add(menuQuitter);
        
        //--------------------------------------------------------------
        
        //On crée un menu "Aide"
        menuAide = new JMenu();
        menuAide.setText("Aide");
        barreMenu.add(menuAide);
        
        //on crée un item "A propos"
        menuAPropos = new JMenuItem();
        menuAPropos.setText("A Propos");
        menuAPropos.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                aPropos();
            }
        });
        menuAide.add(menuAPropos);
        
        
        //--------------------------------------------------------------
        
        //on spécifie la mise en page
        setLayout(new BorderLayout());
        
        
        //On crée le champ de réponses
        ArrayList<URL> liste = new ArrayList<URL>();
        champReponses = new ListeReponses(liste, prefs);
        this.add(champReponses, BorderLayout.CENTER);
        
        //On crée le champ de recherche
        champRecherche = new InterrogerIndex(index, champReponses);
        champRecherche.setBorder(BorderFactory.createTitledBorder(
                BorderFactory.createLineBorder(
                new java.awt.Color(0, 0, 0)), "Recherche"));
        this.add(champRecherche, BorderLayout.NORTH);
        
        //--------------------------------------------------------------
        
        //on signale que l'on a fini d'ajouter des éléments
        pack();
        
        //On change quelques propriétés de la fenêtre
        setTitle("IDXS");
        setSize(800,600);
        setResizable(false);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLocationRelativeTo(null);
        setVisible(true);
    }
    
    /**
     * Permet d'enregistrer l'index
     */
    private void enregistrerIndex(){
        //On crée un JFileChooser
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.addChoosableFileFilter(
                new Filtre("Index de recherche IDXS","idx"));
        fileChooser.showSaveDialog(this);
        
        //On enregistre l'index en utilisant la sérialisation
        File fichier = fileChooser.getSelectedFile();
        if(fichier == null) return;
        String chemin = fichier.getAbsolutePath();
        if(!chemin.endsWith(".idx")){
            chemin += ".idx";
        }
        fichier = new File(chemin);
        try {
            synchronized(this){
                index.enregistrerIndex(fichier);
            }
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
    /**
     * Permet d'ouvrir un index déjà enregistré
     */
    private void ouvrirIndex(){
        selecteur.ouvrir();
    }
    
    /**
     * Affiche un petit message a propos du programme dans un pop-up
     */
    private void aPropos(){
        JOptionPane.showMessageDialog(this,"Programme r\u00e9alis\u00e9 dans le " +
                "cadre du cours\n \"projet de programmation\".\n\n<html><u>" +
                "Professeur:</u></html>\n   P. Wolper\n<html><u>Assistant:" +
                "</u></html>\n   A. Linden\n\n<html><u>Auteurs:</u></html>" +
                "\n   Martin Leon Benjamin\n   Hoessen Benoit\n\n", "A Propos",
                JOptionPane.PLAIN_MESSAGE);
    }
    
    /**
     * Permet d'ajouter des références au sein de l'index
     */
    private void agrandirIndex(){
        CreerIndex tmp = new CreerIndex(index,prefs,this, "Agrandir l'index");
    }
    
    /**
     * Permet d'ajouter des références au sein de l'index
     */
    private void nouvelIndex(){
        synchronized(this){
            changerIndex(new Index());
        }
        CreerIndex tmp = new CreerIndex(index,prefs,this, "Cr\u00e9er un index");
    }
    
    /**
     * Ouvre un pop-up pour modifier les préférences de l'application
     */
    private void preferences(){
        PreferencesPopup tmp = new PreferencesPopup(prefs, this);
    }
    
    /**
     * On quitte l'application, en forçant la fermeture de tous les threads
     * lancés par cette application.
     */
    private void quitter(){
        System.exit(0);
    }
    
    public synchronized void changerIndex(Index index) {
        this.index = index;
        champRecherche.changerInder(index);
    }
    
    private InterrogerIndex champRecherche;
    private Index index;
    private ListeReponses champReponses;
    private JMenuBar barreMenu;
    private JMenuItem menuEnregistrer, menuOuvrir, menuAPropos, menuAgrandir
            ,menuQuitter, menuPreferences, menuNouveau;
    private JMenu menuFichier;
    private Preferences prefs;
    private JMenu menuAide;
    private SelecteurFichier selecteur;
    private static final long serialVersionUID = 20071126L;
    
}
