package be.ulg.hoessen.martinLeon.interfaceGraphique;

import be.ulg.hoessen.martinLeon.chargeurs.ChangeurEntite;
import be.ulg.hoessen.martinLeon.indexeur.Index;
import be.ulg.hoessen.martinLeon.indexeur.IndexException;
import be.ulg.hoessen.martinLeon.Util;
import java.awt.Dimension;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;
import java.util.ArrayList;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

/**
 * Classe permettant d'interroger un index graphiquement
 * @author benjamin, benoit
 */
public class InterrogerIndex extends javax.swing.JPanel {

    /**
     * Crée un nouveau panneau de questionnement d'index
     * @param unIndex l'index que l'on questionnera
     * @param uneListeReponses la liste de réponses que l'on mettra à jour
     */
    public InterrogerIndex(Index unIndex, ListeReponses uneListeReponses) {
        index = unIndex;
        listeReponse = uneListeReponses;
        initialiserComposants();
    }

    /**
     * Permet de changer l'index utilisé
     * @param unIndex le nouvel index que l'on utilisera
     */
    public synchronized void changerInder(Index unIndex) {
        index = unIndex;
    }

    /**
     * On initialise les composants graphiques
     */
    private void initialiserComposants() {
        //on spécifie la mise en page du panneau
        setLayout(new javax.swing.BoxLayout(this,
                javax.swing.BoxLayout.X_AXIS));

        //on ajoute un champ de texte pour que 
        //l'utilisateur puisse entrer sa requête
        champDeRecherche = new javax.swing.JTextField();
        champDeRecherche.setText("Entrez votre recherche ici...");
        champDeRecherche.setMinimumSize(new Dimension(440, 19));
        champDeRecherche.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent evt) {
                appuyerEnter(evt);
            }
        });
        add(champDeRecherche);

        //on ajoute le bouton pour lancer la recherche
        bouttonRechercher = new javax.swing.JButton();
        bouttonRechercher.setText("Rechercher");
        bouttonRechercher.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent evt) {
                lancerRecherche();
            }
        });
        add(bouttonRechercher);

        //on spécifie la taille du panneau
        setMinimumSize(new Dimension(600, 40));
    }

    private void appuyerEnter(KeyEvent evt) {
        if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
            lancerRecherche();
        }
    }

    private void lancerRecherche() {
        //on met tout en minuscules
        String entree = champDeRecherche.getText().toLowerCase();
        if (entree.length() > 0) {
            //on enlève les accents
            StringBuffer question = new StringBuffer();
            for (int i = 0; i < entree.length(); i++) {
                question.append(
                        ChangeurEntite.caractereCorrespondant(entree.charAt(i)));
            }

            ArrayList<URL> liste = null;
            try {
                synchronized (this) {
                    liste = index.resultat(question.toString());
                }
                listeReponse.changerListeReponses(liste);
                JOptionPane.showMessageDialog(null, "Nombre de r\u00e9sultats" +
                        " trouv\u00e9s: "+liste.size());
            } catch (IndexException ex) {
                //ex.printStackTrace();
                Util.erreur(ex.toString());
                return;
            }
        }
        return;
    }
    private Index index;
    private JButton bouttonRechercher;
    private JTextField champDeRecherche;
    private static final long serialVersionUID = 20071127L;
    private ListeReponses listeReponse;
}
