package be.ulg.hoessen.martinLeon.interfaceGraphique;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;

/**
 * Cette classe represente un panneau permettant d'afficher dix réponses à la
 * fois, avec des boutons pour la navigation
 * @author benjamin, benoit
 */
//TODO: modifier l'utilisation de cette classe pour qu'elle n'ait que 10 objets réponses enregistrés
public class ListeReponses extends JPanel {

    /**
     * Crée une nouvelle instance de ListeReponses
     * @param uneListe la liste des réponses a afficher
     */
    public ListeReponses(ArrayList<URL> uneListe, Preferences p) {
        prefs = p;
        initialiserComposants();
        changerListe(uneListe);
    }

    /**
     * On change la liste de réponses
     * @param uneListe la nouvelle liste de réponse
     */
    private void changerListe(ArrayList<URL> uneListe) {
        synchronized (this) {
            liste = uneListe;
            premierItem = 0;
        }
        chargerReponses();
    }

    /**
     * On change la liste de réponses
     * @param uneListe la nouvelle liste d'URL
     */
    public void changerListeReponses(ArrayList<URL> uneListe) {
        ArrayList<URL> tmp = new ArrayList<URL>(uneListe.size());
        for (int i = 0; i < uneListe.size(); i++) {
            tmp.add(uneListe.get(i));
        }
        changerListe(tmp);
    }

    /**
     * Permet d'initaliser les composants
     */
    private synchronized void initialiserComposants() {
        //on spécifie la mise en page du panneau
        setLayout(new BorderLayout());

        //on crée le panneau qui contiendra les boutons précédent et suivants
        boutons = new JPanel();
        boutons.setLayout(new GridLayout(1, 3));

        //on crée le bouton poru afficher les résultats précédents
        precedent = new JButton();
        precedent.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent evt) {
                precedent();
            }
        });
        precedent.setText("<< R\u00e9sultats pr\u00e9c\u00e9dents");
        boutons.add(precedent);

        //On laisse un peu d'espace vide
        boutons.add(new JPanel());

        //On crée le bouton pour afficher les résultats suivants
        suivant = new JButton();
        suivant.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent evt) {
                suivant();
            }
        });
        suivant.setText("R\u00e9sultats suivants >>");
        boutons.add(suivant);

        //On ajoute le panneau de boutons dans le bas
        add(boutons, BorderLayout.SOUTH);

        //On crée le panneau qui contiendra les résultats
        resultPanel = new JPanel();
        resultPanel.setLayout(new GridLayout(10, 1));

        reponses = new Reponse[NB_REPONSES];
        for (int i = 0; i < reponses.length; i++) {
            reponses[i] = new Reponse(null, prefs);
            reponses[i].visible(false);
            resultPanel.add(reponses[i]);
        }

        //On crée un joli bord autour du panneau
        resultPanel.setBorder(BorderFactory.createTitledBorder(
                BorderFactory.createLineBorder(
                new Color(0, 0, 0)), "R\u00e9sultats"));
        resultPanel.setMinimumSize(new Dimension(610, 400));
        add(resultPanel, BorderLayout.CENTER);



        //on spécifie la taille du panneau
        setMinimumSize(new Dimension(650, 500));
    }

    /**
     * On affiche les résultats précédents
     */
    private void precedent() {
        if (premierItem > 9) {
            synchronized (this) {
                premierItem -= 10;
            }
            chargerReponses();
        }
    }

    /**
     * on affiche les résultats suivants
     */
    private void suivant() {
        if (premierItem < liste.size() - 10) {
            synchronized (this) {
                premierItem += 10;
            }
            chargerReponses();
        }
    }

    /**
     * Charge et affiche les dix prochaines réponses a partir de debut
     */
    private synchronized void chargerReponses() {
        //on enleve les resultats precedents
        //resultPanel.removeAll();
        //on ajoute les nouveaux
        for (int i = 0; i < 10; i++) {
            if (i + premierItem < liste.size()) {
                reponses[i].changerURL(liste.get(i + premierItem));
                reponses[i].visible(true);
            } else {
                reponses[i].visible(false);
            }
        }

        //on valide notre affichage
        resultPanel.validate();
        //on met a jour l'interface
        resultPanel.updateUI();
        //On affiche les boutons si necessaire
        suivant.setEnabled(premierItem + 10 < liste.size());
        precedent.setEnabled(premierItem >= 10);
    }
    private JPanel boutons;
    private JButton precedent;
    private JPanel resultPanel;
    private JButton suivant;
    /**  La liste des réponses qui ont ete fournies                  */
    private Reponse[] reponses;
    private ArrayList<URL> liste;
    /**  Le numéro de la premiere reponse a afficher                 */
    private int premierItem;
    private static final int NB_REPONSES = 10;
    private static final long serialVersionUID = 1L;
    private Preferences prefs;
}
