package be.ulg.hoessen.martinLeon.interfaceGraphique;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.HashMap;
import javax.swing.filechooser.FileSystemView;

/**
 * Classe permettant de gérer les préférences de l'utilisateur. Pour ouvrir
 * les préférences, on doit utiliser la méthode charger() qui va voir si aucun
 * fichier de configuration existe. Si il existe, on le récupère, sinon on le 
 * créera. 
 * @author benjamin, benoit
 */
public class Preferences implements Serializable{
    
    /** nom du fichier qui sera dans le répertoire utilisateur            */
    private static final String fichierConf = ".idsx.conf";
    /** la liste des preferences                                          */
    private HashMap<String,Object> pref;
    /** numéro de version pour la sérialisation                           */
    private final static long serialVersionUID = 20071115;
    /** l'objet qui permet de savoir qu'aucune préférence n'a été défnine */
    public static final Object DEFAULT = new Object();
    /** Quelques nom de préférences */
    public static final String NB_ROBOTS = "nbRobots";
    public static final String NAVIGATEUR = "navigateur";
    
    /**
     * Cree une nouvelle instance de Preferences. Le constructeur est privé car
     * on doit à chaque fois regarder si le fichier de configuration n'existe
     * pas déjà.
     */
    private Preferences() {
        pref = new HashMap<String, Object>();
    }
    
    /**
     * Permet de modifier une préférence.
     * @param description, le nom de la préférence
     * @param valeur, la valeur de la préférence
     */
    public void modifierPreference(String description, Object valeur){
        pref.put(description, valeur);
    }
    
    /**
     * Permet de charger la valeur d'une préférence
     * @param description, le nom de la préférence
     * @return la valeur de la préférence si elle existe, 
     * Preferences.DEFAULT sinon
     */
    public Object valeur(String description){
        if (pref.containsKey(description)){
            return pref.get(description);
        } else return DEFAULT;
    }
    
    /**
     * Permet d'enregistrer les préférences
     */
    public void enregistrer() throws IOException{
        //on ouvre un fichier fichierConf dans le répertoire de l'utilisateur
        File fichier = new File(
                FileSystemView.getFileSystemView().getDefaultDirectory(),
                fichierConf);
        ObjectOutputStream out = new ObjectOutputStream(
                new FileOutputStream(fichier));
        out.writeObject(this);
    }
    
    /**
     * Permet de charger les préférences déjà enregistrées. Si elles n'existent
     * pas, de nouvelles préférences sont crées.
     */
    public static Preferences charger(){
        Preferences p = null;
        //On ouvre le fichier fichierConf dans le répertoire de l'utilisateur
        File fichier = new File(
                FileSystemView.getFileSystemView().getDefaultDirectory(),
                fichierConf);
        try {
            ObjectInputStream in = new ObjectInputStream(
                    new FileInputStream(fichier));
            Object a = in.readObject();
            if(a instanceof Preferences){
                p = (Preferences)a;
            }
        } catch (Exception ex) {
            //Le fichier de configuration n'existe pas. On en crée un
            p = new Preferences();
        }
        return p;
    }
}
