package be.ulg.hoessen.martinLeon.interfaceGraphique;

import be.ulg.hoessen.martinLeon.Util;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.GridLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
/**
 * Le but de cette fenêtre est de pouvoir modifier les préférences de
 * l'utilisateur: le navigateur internet et le nombre de robots qui parcourent
 * le web.
 * @author  benjamin, benoit
 */
public class PreferencesPopup extends JFrame {
    
    /**
     *
     */
    public PreferencesPopup(Preferences preferences, Component parent) {
        composentParent = parent;
        prefs = preferences;
        initialiserComposants();
    }
    
    private void initialiserComposants() {
        
        getContentPane().setLayout(new BorderLayout());
        
        //on crée le panneau central qui contiendra les champs que l'on peut
        //modifier
        panneauCentral = new JPanel();
        panneauCentral.setLayout(new GridLayout(3, 2, 4, 2));
        getContentPane().add(panneauCentral,BorderLayout.CENTER);
        
        //on crée un label pour donner des informations à l'utilisateur
        labelNbRobots = new JLabel();
        labelNbRobots.setText("Nombre de robots");
        panneauCentral.add(labelNbRobots);
        
        //on crée le champ qui contiendra le nombre de robots que l'on veut
        //utiliser
        nbRobots = new JTextField();
        nbRobots.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent evt) {
                if(evt.getKeyCode() == KeyEvent.VK_ENTER){
                    modifierPreferences();
                }
            }
        });
        //on va voir la préférence de l'utilisateur qui a déjà été introduite
        Object prefNbRobots = prefs.valeur(prefs.NB_ROBOTS);
        if(prefNbRobots == Preferences.DEFAULT){
            nbRobots.setText("15");
        } else {
            nbRobots.setText(prefNbRobots.toString());
        }
        panneauCentral.add(nbRobots);
        
        //on crée un label pour informer l'utilisateur
        labelNavigateur = new JLabel();
        labelNavigateur.setText("Commande pour le navigateur");
        panneauCentral.add(labelNavigateur);
        
        //On crée un panneau qui contiendra les deux méthode pour spécifier
        //le navigateur que l'on va utiliser. Soit en entrant la commande, 
        //soit en entrant le chemin vers l'exécutable
        JPanel choixNav = new JPanel();
        choixNav.setLayout(new GridLayout(1,2));
        
        //champ de texte que où l'on peut entrer la commande pour lancer
        //le navigateur
        navigateur = new JTextField();
        navigateur.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent evt) {
                if(evt.getKeyCode() == KeyEvent.VK_ENTER){
                    modifierPreferences();
                }
            }
        });
        //On va voir la valeur par défaut
        Object prefNavigateur = prefs.valeur(prefs.NAVIGATEUR);
        if(prefNavigateur == Preferences.DEFAULT){
            navigateur.setText("firefox");
        } else {
            navigateur.setText(prefNavigateur.toString());
        }
        choixNav.add(navigateur);
        JButton bouttonParcourir = new JButton();
        
        //Bouton pour parcourir et chercher l'exécutable du navigateur
        bouttonParcourir.setText("Parcourir");
        bouttonParcourir.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt) {
                chercherNavigateur();
            }
        });
        choixNav.add(bouttonParcourir);
        panneauCentral.add(choixNav);
        
        //on crée un bouton qui nous permettra de valider les modifications
        bouttonOk = new JButton();
        bouttonOk.setText("OK");
        bouttonOk.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt) {
                modifierPreferences();
            }
        });
        //On crée un panneau pour ajouter le bouton pour qu'il soit centré
        JPanel tmp = new JPanel();
        tmp.add(bouttonOk);
        getContentPane().add(tmp,BorderLayout.SOUTH);
        
        //on laisse un peu d'espace autour des éléments
        getContentPane().add(new JPanel(),BorderLayout.EAST);
        getContentPane().add(new JPanel(),BorderLayout.WEST);
        getContentPane().add(new JPanel(),BorderLayout.NORTH);
        
        //on valide l'insertion des éléments
        pack();
        
        //on spécifie quelques propriété de la fenêtre.
        setResizable(false);
        setTitle("Modifier les pr\u00e9f\u00e9rences");
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setSize(500,150);
        setLocationRelativeTo(composentParent);
        setVisible(true);
    }
    
    /**
     * Permet de valider et d'enregistrer les entrées saisies
     */
    private void modifierPreferences() {
        try {
            prefs.modifierPreference(prefs.NB_ROBOTS,
                    new Integer(Integer.parseInt(nbRobots.getText())));
            prefs.modifierPreference(prefs.NAVIGATEUR,navigateur.getText());
            prefs.enregistrer();
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (NumberFormatException ex) {
            ex.printStackTrace();
            Util.erreur("Vous avez entr\u00e9 un nombre négatif pour le" +
                    " nombre de robot. Le nombre par d\u00e9faut" +
                    " (15) est pris.");
        }
        dispose() ;
    }
    
    /**
     * Ouvre un navigateur pour aller chercher l'executable pour lancer le
     * navigateur
     */
    private void chercherNavigateur(){
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.showOpenDialog(this);
        navigateur.setText(fileChooser.getSelectedFile().getAbsolutePath());
    }
    
    
    private JButton bouttonOk;
    private JComboBox jComboBox1;
    private JComboBox jComboBox2;
    private JLabel labelNbRobots;
    private JLabel labelNavigateur;
    private JPanel panneauCentral;
    private JTextField nbRobots;
    private Preferences prefs;
    private final static long serialVersionUID = 20071116;
    private JTextField navigateur;
    private Component composentParent;
}