package be.ulg.hoessen.martinLeon.interfaceGraphique;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.net.URL;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;

/**
 * Panneau représentant une réponse. On y voit un label qui représente l'adresse
 * de la page et un bouton [Visiter]. Si l'on clique sur l'un ou l'autre, on
 * ouvre le navigateur pour visiter cette page.
 * @author benjamin, benoit
 */
public class Reponse extends javax.swing.JPanel {

    /** bouton pour visiter le lien                     */
    private JButton visiter;
    /** label représentant l'adresse du lien            */
    private JLabel champUrl;
    /** le lien                                         */
    private URL url;
    /**
     * les balises html à appliquer au lien pour que celui-ci soit souligné dans
     * le label
     */
    private static String souligner = "<html><u>",  finSouligner = "</u></html>";
    private Preferences prefs;
    /** numéro de version pour la sérialisation         */
    private static final long serialVersionUID = 1L;

    /**
     * Crée une nouvelle réponse qui pointe vers une url spécifié
     * @param uneURL, l'url que l'on a reçu en réponse
     */
    public Reponse(URL uneURL, Preferences p) {
        prefs = p;
        url = uneURL;
        initialiserComposants();
    }

    /**
     * Permet d'initialiser les composants graphiques
     */
    private void initialiserComposants() {

        champUrl = new JLabel();
        final Reponse thisReponse = this;
        //On ajoute un mouseListener qui changera le pointeur de souris au
        //survol du label, et lors d'un clic, on visitera le lien
        champUrl.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseEntered(MouseEvent evt) {
                if (!champUrl.getText().equals("")) {
                    thisReponse.setCursor(
                            Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                }
            }

            @Override
            public void mouseExited(MouseEvent evt) {
                thisReponse.setCursor(Cursor.getDefaultCursor());
            }

            @Override
            public void mouseClicked(MouseEvent evt) {
                if (!champUrl.getText().equals("")) {
                    visiterLien();
                }
            }
        });
        //On applique différents attributs au label
        if (url != null) {
            String tmp = url.toString();
            if (tmp.length() > 80) {
                tmp = tmp.substring(0, 80) + "...";
            }
            champUrl.setText(souligner + tmp + finSouligner);
        } else {
            champUrl.setText("");
        }
        champUrl.setHorizontalAlignment(JLabel.LEFT);
        champUrl.setForeground(new Color(0, 0, 255));
        champUrl.setMinimumSize(new Dimension(20, 10));
        champUrl.setBorder(new SoftBevelBorder(BevelBorder.RAISED));

        //On crée le bouton
        visiter = new JButton();
        visiter.setMinimumSize(new Dimension(20, 10));
        visiter.setText("Visiter");
        //On ajoute un écouteur d'évènement sur le bouton pour que lorsque l'on
        //clique dessus, on visite le lien
        visiter.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent evt) {
                visiterLien();
            }
        });

        //On choisit la mise en page du panneau
        BorderLayout blayout = new BorderLayout();
        blayout.setHgap(15);
        setLayout(blayout);
        //On ajoute les éléments
        add(visiter, BorderLayout.WEST);
        add(champUrl, BorderLayout.CENTER);
        //On ajoute des panneaux vide pour avoir un peu d'espace
        add(new JPanel(), BorderLayout.NORTH);
        add(new JPanel(), BorderLayout.SOUTH);

        //On spécifie la taille du panneau
        setMaximumSize(new Dimension(600, 30));
        setMinimumSize(new Dimension(600, 30));
        setPreferredSize(new Dimension(600, 30));
    }

    /**
     * Permet de changer l'url représentée
     * @param URL l'url que l'on doit représenter
     */
    public void changerURL(URL url) {
        String tmp = url.toString();
        if (tmp.length() > 80) {
            tmp = tmp.substring(0, 80) + "...";
        }
        champUrl.setText(souligner + tmp + finSouligner);
        this.url = url;
        champUrl.validate();
        this.validate();
    }

    public void visible(boolean b) {
        if (b) {
            champUrl.setText(url.toString());
            visiter.setEnabled(true);
        } else {
            champUrl.setText("");
            visiter.setEnabled(false);
        }
        champUrl.validate();
        visiter.validate();
        this.validate();
    }

    /**
     * On ouvre le navigateur
     */
    private void visiterLien() {
        try {
            String navigateur;
            Object prefNavigateur = prefs.valeur(Preferences.NAVIGATEUR);
            if (prefNavigateur == Preferences.DEFAULT) {
                navigateur = "firefox";
            } else {
                navigateur = prefNavigateur.toString();
            }
            Runtime.getRuntime().exec(new StringBuffer(navigateur).append(' ').append(url.toString()).toString());
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
}
