package be.ulg.hoessen.martinLeon.interfaceGraphique;

import be.ulg.hoessen.martinLeon.Util;
import be.ulg.hoessen.martinLeon.indexeur.Index;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileSystemView;

/**
 * Cette classe permet d'ouvrir via une interface graphique un fichier idx
 * enregistré sur le disque. Il possède en plus un panneau dans lequel on
 * inscrit une description du fichier idx selectionné.
 * @author benjamin, benoit
 */
public class SelecteurFichier extends JFrame {
    
    /**
     * Crée un nouveau selecteur de fichier idx lié à la fenêtre spécifiée
     * @param a la fenêtre qui doit ouvrir des fichiers index.
     */
    public SelecteurFichier(Fenetre a) {
        fenetre = a;
        this.choixEffectue = false;
        initialiserComposants();
        chooser.addPropertyChangeListener(
                JFileChooser.SELECTED_FILE_CHANGED_PROPERTY,
                new PropertyChangeListener(){
            public void propertyChange(PropertyChangeEvent evt) {
                montrerDescription();
            }
        });
    }
    
    /**
     * Permet de montrer la description du fichier idx selectionné.
     */
    private void montrerDescription(){
        File fichier = chooser.getSelectedFile();
        if(fichier !=null && !fichier.isDirectory()){
            try {
                description.setText(Index.chargerDescription(fichier));
            } catch (IOException ex) {
                description.setText("Fichier corrompu.");
                ex.printStackTrace();
            }
        }
    }
    
    /**
     * initialise les différents composants
     */
    private void initialiserComposants() {
        JPanel tmp = new JPanel();
        chooser = new JFileChooser();
        chooser.addChoosableFileFilter(
                new Filtre("Index de recherche IDXS","idx"));
        chooser.setAcceptAllFileFilterUsed(false);
        chooser.addPropertyChangeListener(
                JFileChooser.SELECTED_FILE_CHANGED_PROPERTY,
                new PropertyChangeListener(){
            public void propertyChange(PropertyChangeEvent evt) {
                montrerDescription();
            }
        });
        
        chooser.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                if(e.getActionCommand().equals("CancelSelection")){
                    annuler();
                } if (e.getActionCommand().equals("ApproveSelection")){
                    valider();
                }
                
            }
        });
        
        panneauDescription = new JPanel();
        panneauGlissant = new JScrollPane();
        description = new JTextArea();
        description.setBackground(
                UIManager.getDefaults().getColor("Panel.background"));
        
        tmp.setLayout(new BoxLayout(tmp, BoxLayout.X_AXIS));
        
        tmp.add(chooser);
        
        panneauDescription.setLayout(new GridLayout(1, 0));
        
        panneauDescription.setBorder(BorderFactory.createTitledBorder(
                BorderFactory.createLineBorder(new Color(0, 0, 0)),
                "Description", TitledBorder.DEFAULT_JUSTIFICATION,
                TitledBorder.DEFAULT_POSITION,
                new Font("Dialog", 0, 11), new Color(0, 0, 0)));
        //panneauGlissant.setHorizontalScrollBarPolicy(
        //        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        description.setColumns(20);
        description.setRows(5);
        description.setFocusable(false);
        panneauGlissant.setViewportView(description);
        
        panneauDescription.add(panneauGlissant);
        
        tmp.add(panneauDescription);
        
        this.add(new JPanel().add(tmp));
        
        this.addWindowListener(new WindowListener(){
            public void windowOpened(WindowEvent e) {}
            public void windowClosing(WindowEvent e) {
                annuler();
            }
            public void windowClosed(WindowEvent e) {
                annuler();
            }
            public void windowIconified(WindowEvent e) {}
            public void windowDeiconified(WindowEvent e) {}
            public void windowActivated(WindowEvent e) {}
            public void windowDeactivated(WindowEvent e) {}
        });
        
        //setDefaultCloseOperation(this.EXIT_ON_CLOSE);
        setSize(600,400);
        setTitle("Ouvrir un fichier...");
        pack();
        setVisible(false);
    }
    
    /**
     * Commande à utiliser pour rendre le SelecteurFichier visible. Ainsi, on
     * peut l'utiliser plusieurs fois de suite sans en instancier un nouveau.
     */
    public void ouvrir(){
        if(dernierSelectionne!=null){
            chooser.changeToParentDirectory();
            if (dernierSelectionne.isDirectory()){
                chooser.setCurrentDirectory(dernierSelectionne);
            } else {
                chooser.setCurrentDirectory(dernierSelectionne.getParentFile());
            }
        } else {
            chooser.changeToParentDirectory();
            chooser.setCurrentDirectory(
                    FileSystemView.getFileSystemView().getDefaultDirectory());
        }
        setLocationRelativeTo(fenetre);
        setVisible(true);
    }
    
    /**
     * Permet de lancer les différents traitements lorsque l'on valide le choix
     */
    private synchronized void valider(){
        if(!chooser.getSelectedFile().isDirectory()){
            choixEffectue = true;
            dernierSelectionne = chooser.getSelectedFile();
            setVisible(false);
        }
        if(fenetre!=null){
            try {
                fenetre.changerIndex(Index.chargerIndex(dernierSelectionne));
            } catch (Exception ex) {
                Util.erreur(ex.toString());
                ex.printStackTrace();
            }
            //dispose();
        }
    }
    
    /**
     * Permet de lancer les différentes commandes lorsque l'on annule ou que
     * l'on quitte la fenêtre.
     */
    private synchronized void annuler(){
        choixEffectue = true;
        dernierSelectionne = chooser.getSelectedFile();
        setVisible(false);
        if(fenetre!=null){
            //dispose();
        }
    }
    
    public static File obtenirFichierIDX(){
        SelecteurFichier tmp = new SelecteurFichier(null);
        while(!tmp.choixEffectue){
            
        }
        File f = tmp.chooser.getSelectedFile();
        tmp.dispose();
        return f;
    }
    
    private JFileChooser chooser;
    private JTextArea description;
    private JScrollPane panneauGlissant;
    private JPanel panneauDescription;
    private static final long serialVersionUID = 20071130L;
    private volatile boolean choixEffectue;
    private Fenetre fenetre;
    private File dernierSelectionne;
    
    public static void main(String args[]){
        System.out.println(SelecteurFichier.obtenirFichierIDX().getAbsolutePath());
    }
    
}
